=== modified file '.bzrignore'
--- .bzrignore	2007-04-12 20:27:42 +0000
+++ .bzrignore	2007-05-09 15:36:06 +0000
@@ -34,3 +34,4 @@
 ./html_docs
 ./pretty_docs
 ./api
+doc/**/*.htm

=== modified file 'Makefile'
--- Makefile	2007-04-30 18:42:34 +0000
+++ Makefile	2007-05-08 04:14:01 +0000
@@ -75,6 +75,8 @@
 doc_dir := doc 
 txt_files := $(wildcard $(addsuffix /*.txt, $(doc_dir))) doc/bzr_man.txt
 htm_files := $(patsubst %.txt, %.htm, $(txt_files)) 
+dev_txt_files := $(wildcard $(addsuffix /*.txt, doc/developers))
+dev_htm_files := $(patsubst %.txt, %.htm, $(dev_txt_files)) 
 
 pretty-html-docs: pretty_files
 
@@ -83,11 +85,14 @@
 
 pretty_files: $(patsubst doc/%.txt, $(PRETTYDIR)/%.htm, $(txt_files))
 
-doc/HACKING.htm: HACKING
-	python tools/rst2html.py --link-stylesheet --stylesheet=default.css HACKING doc/HACKING.htm
-
-doc/%.htm: doc/%.txt 
-	python tools/rst2html.py --link-stylesheet --stylesheet=default.css doc/$*.txt doc/$*.htm
+doc/developers/%.htm: doc/developers/%.txt
+	python tools/rst2html.py --link-stylesheet --stylesheet=../default.css $< $@
+
+doc/developers/HACKING.htm: doc/developers/HACKING
+	python tools/rst2html.py --link-stylesheet --stylesheet=../default.css $< $@
+
+%.htm: %.txt
+	python tools/rst2html.py --link-stylesheet --stylesheet=default.css $< $@
 
 $(PRETTYDIR)/%.htm: pretty_docs doc/%.txt
 	python tools/rst2prettyhtml.py doc/bazaar-vcs.org.kid doc/$*.txt \
@@ -108,17 +113,20 @@
 man1/bzr.1: $(MAN_DEPENDENCIES)
 	python generate_docs.py -o $@ man
 
-docs: $(htm_files) $(MAN_PAGES) doc/HACKING.htm
+ALL_DOCS = $(htm_files) $(MAN_PAGES) doc/developers/HACKING.htm $(dev_htm_files)
+docs: $(ALL_DOCS)
 
 copy-docs: docs
 	python tools/win32/ostools.py copytodir $(htm_files) \
-		doc/default.css doc/HACKING.htm NEWS README \
+		doc/default.css NEWS README \
 		win32_bzr.exe/doc
+	python tools/win32/ostools.py copytodir doc/developers/HACKING.htm \
+		win32_bzr.exe/doc/developers
 
 # clean produced docs
 clean-docs:
-	python tools/win32/ostools.py remove $(htm_files) \
-	$(HTMLDIR) $(PRETTYDIR) doc/bzr_man.txt $(MAN_PAGES)
+	python tools/win32/ostools.py remove $(ALL_DOCS) \
+	$(HTMLDIR) $(PRETTYDIR) doc/bzr_man.txt
 
 
 # make bzr.exe for win32 with py2exe

=== modified file 'NEWS'
--- NEWS	2007-04-30 07:48:55 +0000
+++ NEWS	2007-05-10 05:55:01 +0000
@@ -1,5 +1,37 @@
 IN DEVELOPMENT
 
+  BUGFIXES:
+
+    * ``bzr push`` should only connect to the remote location one time.
+      We have been connecting 3 times because we forget to pass around
+      the Transport object. This adds ``BzrDir.clone_on_transport()``, so
+      that we can pass in the Transport that we already have.
+      (John Arbash Meinel, #75721)
+
+
+bzr 0.16  2007-05-07
+  
+  BUGFIXES:
+
+    * Handle when you have 2 directories with similar names, but one has a
+      hyphen. (``'abc'`` versus ``'abc-2'``). The WT4._iter_changes
+      iterator was using direct comparison and ``'abc/a'`` sorts after
+      ``'abc-2'``, but ``('abc', 'a')`` sorts before ``('abc-2',)``.
+      (John Arbash Meinel, #111227)
+
+    * Handle when someone renames a file on disk without telling bzr.
+      Previously we would report the first file as missing, but not show
+      the new unknown file. (John Arbash Meinel, #111288)
+
+    * Avoid error when running hooks after pulling into or pushing from
+      a branch bound to a smartserver branch.  (Martin Pool, #111968)
+
+  IMPROVEMENTS:
+
+    * Move developer documentation to doc/developers/. This reduces clutter in
+      the root of the source tree and allows HACKING to be split into multiple
+      files. (Robert Collins, Alexander Belchenko)
+
 bzr 0.16rc2  2007-04-30
 
   BUGFIXES:

=== modified file 'README'
--- README	2007-03-21 04:28:02 +0000
+++ README	2007-05-08 20:03:44 +0000
@@ -1,6 +1,6 @@
-=====================================
-README for Bazaar DEVELOPMENT VERSION
-=====================================
+==========================
+README for Bazaar 0.17-dev
+==========================
 
 Bazaar is a decentralized revision control system, designed to be easy
 for developers and end users alike.

=== modified file 'bzr'
--- bzr	2007-04-17 04:39:12 +0000
+++ bzr	2007-05-08 20:03:44 +0000
@@ -79,7 +79,8 @@
 import bzrlib.decorators
 if ('--lsprof' in sys.argv
     or '--lsprof-file' in sys.argv
-    or '--profile' in sys.argv):
+    or '--profile' in sys.argv
+    or '--lsprof-timed' in sys.argv):
     bzrlib.decorators.use_pretty_decorators()
 else:
     bzrlib.decorators.use_fast_decorators()
@@ -87,7 +88,7 @@
 import bzrlib.commands
 import bzrlib.trace
 
-if bzrlib.version_info[:3] != (0, 16, 0):
+if bzrlib.version_info[:3] != (0, 17, 0):
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"
             "bzrlib from %s is version %r\n"

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2007-04-01 06:19:16 +0000
+++ bzrlib/__init__.py	2007-05-08 20:03:44 +0000
@@ -35,7 +35,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (0, 16, 0, 'dev', 0)
+version_info = (0, 17, 0, 'dev', 0)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-05-01 16:11:36 +0000
+++ bzrlib/branch.py	2007-05-14 17:21:02 +0000
@@ -783,10 +783,7 @@
         :return: The tree of the created checkout
         """
         t = transport.get_transport(to_location)
-        try:
-            t.mkdir('.')
-        except errors.FileExists:
-            pass
+        t.ensure_base()
         if lightweight:
             format = self._get_checkout_format()
             checkout = format.initialize_on_transport(t)
@@ -1006,7 +1003,7 @@
         # (push_result)
         # containing the members
         # (source, local, master, old_revno, old_revid, new_revno, new_revid)
-        # where local is the local branch or None, master is the target 
+        # where local is the local target branch or None, master is the target 
         # master branch, and the rest should be self explanatory. The source
         # is read locked and the target branches write locked. Source will
         # be the local low-latency branch.
@@ -1482,13 +1479,14 @@
 
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-        _hook_master=None, _run_hooks=True):
+             _hook_master=None, run_hooks=True):
         """See Branch.pull.
 
         :param _hook_master: Private parameter - set the branch to 
             be supplied as the master to push hooks.
-        :param _run_hooks: Private parameter - allow disabling of
-            hooks, used when pushing to a master branch.
+        :param run_hooks: Private parameter - if false, this branch
+            is being called because it's the master of the primary branch,
+            so it should not run its hooks.
         """
         result = PullResult()
         result.source_branch = source
@@ -1513,7 +1511,7 @@
             else:
                 result.master_branch = self
                 result.local_branch = None
-            if _run_hooks:
+            if run_hooks:
                 for hook in Branch.hooks['post_pull']:
                     hook(result)
         finally:
@@ -1531,40 +1529,91 @@
 
     @needs_read_lock
     def push(self, target, overwrite=False, stop_revision=None,
-        _hook_master=None, _run_hooks=True):
+             _override_hook_source_branch=None):
         """See Branch.push.
+
+        This is the basic concrete implementation of push()
+
+        :param _override_hook_source_branch: If specified, run
+        the hooks passing this Branch as the source, rather than self.  
+        This is for use of RemoteBranch, where push is delegated to the
+        underlying vfs-based Branch. 
+        """
+        # TODO: Public option to disable running hooks - should be trivial but
+        # needs tests.
+        target.lock_write()
+        try:
+            result = self._push_with_bound_branches(target, overwrite,
+                    stop_revision,
+                    _override_hook_source_branch=_override_hook_source_branch)
+            return result
+        finally:
+            target.unlock()
+
+    def _push_with_bound_branches(self, target, overwrite,
+            stop_revision,
+            _override_hook_source_branch=None):
+        """Push from self into target, and into target's master if any.
         
-        :param _hook_master: Private parameter - set the branch to 
-            be supplied as the master to push hooks.
-        :param _run_hooks: Private parameter - allow disabling of
-            hooks, used when pushing to a master branch.
+        This is on the base BzrBranch class even though it doesn't support 
+        bound branches because the *target* might be bound.
+        """
+        def _run_hooks():
+            if _override_hook_source_branch:
+                result.source_branch = _override_hook_source_branch
+            for hook in Branch.hooks['post_push']:
+                hook(result)
+
+        bound_location = target.get_bound_location()
+        if bound_location and target.base != bound_location:
+            # there is a master branch.
+            #
+            # XXX: Why the second check?  Is it even supported for a branch to
+            # be bound to itself? -- mbp 20070507
+            master_branch = target.get_master_branch()
+            master_branch.lock_write()
+            try:
+                # push into the master from this branch.
+                self._basic_push(master_branch, overwrite, stop_revision)
+                # and push into the target branch from this. Note that we push from
+                # this branch again, because its considered the highest bandwidth
+                # repository.
+                result = self._basic_push(target, overwrite, stop_revision)
+                result.master_branch = master_branch
+                result.local_branch = target
+                _run_hooks()
+                return result
+            finally:
+                master_branch.unlock()
+        else:
+            # no master branch
+            result = self._basic_push(target, overwrite, stop_revision)
+            # TODO: Why set master_branch and local_branch if there's no
+            # binding?  Maybe cleaner to just leave them unset? -- mbp
+            # 20070504
+            result.master_branch = target
+            result.local_branch = None
+            _run_hooks()
+            return result
+
+    def _basic_push(self, target, overwrite, stop_revision):
+        """Basic implementation of push without bound branches or hooks.
+
+        Must be called with self read locked and target write locked.
         """
         result = PushResult()
         result.source_branch = self
         result.target_branch = target
-        target.lock_write()
+        result.old_revno, result.old_revid = target.last_revision_info()
         try:
-            result.old_revno, result.old_revid = target.last_revision_info()
-            try:
-                target.update_revisions(self, stop_revision)
-            except DivergedBranches:
-                if not overwrite:
-                    raise
-            if overwrite:
-                target.set_revision_history(self.revision_history())
-            result.tag_conflicts = self.tags.merge_to(target.tags)
-            result.new_revno, result.new_revid = target.last_revision_info()
-            if _hook_master:
-                result.master_branch = _hook_master
-                result.local_branch = target
-            else:
-                result.master_branch = target
-                result.local_branch = None
-            if _run_hooks:
-                for hook in Branch.hooks['post_push']:
-                    hook(result)
-        finally:
-            target.unlock()
+            target.update_revisions(self, stop_revision)
+        except DivergedBranches:
+            if not overwrite:
+                raise
+        if overwrite:
+            target.set_revision_history(self.revision_history())
+        result.tag_conflicts = self.tags.merge_to(target.tags)
+        result.new_revno, result.new_revid = target.last_revision_info()
         return result
 
     def get_parent(self):
@@ -1640,11 +1689,12 @@
         
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-        _run_hooks=True):
-        """Extends branch.pull to be bound branch aware.
+             run_hooks=True):
+        """Pull from source into self, updating my master if any.
         
-        :param _run_hooks: Private parameter used to force hook running
-            off during bound branch double-pushing.
+        :param run_hooks: Private parameter - if false, this branch
+            is being called because it's the master of the primary branch,
+            so it should not run its hooks.
         """
         bound_location = self.get_bound_location()
         master_branch = None
@@ -1656,33 +1706,10 @@
             if master_branch:
                 # pull from source into master.
                 master_branch.pull(source, overwrite, stop_revision,
-                    _run_hooks=False)
+                    run_hooks=False)
             return super(BzrBranch5, self).pull(source, overwrite,
                 stop_revision, _hook_master=master_branch,
-                _run_hooks=_run_hooks)
-        finally:
-            if master_branch:
-                master_branch.unlock()
-
-    @needs_read_lock
-    def push(self, target, overwrite=False, stop_revision=None):
-        """Updates branch.push to be bound branch aware."""
-        bound_location = target.get_bound_location()
-        master_branch = None
-        if bound_location and target.base != bound_location:
-            # not pushing to master, so we need to update master.
-            master_branch = target.get_master_branch()
-            master_branch.lock_write()
-        try:
-            if master_branch:
-                # push into the master from this branch.
-                super(BzrBranch5, self).push(master_branch, overwrite,
-                    stop_revision, _run_hooks=False)
-            # and push into the target branch from this. Note that we push from
-            # this branch again, because its considered the highest bandwidth
-            # repository.
-            return super(BzrBranch5, self).push(target, overwrite,
-                stop_revision, _hook_master=master_branch)
+                run_hooks=run_hooks)
         finally:
             if master_branch:
                 master_branch.unlock()

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-05-01 16:11:36 +0000
+++ bzrlib/builtins.py	2007-05-14 17:21:02 +0000
@@ -709,7 +709,6 @@
                 location = stored_loc
 
         to_transport = transport.get_transport(location)
-        location_url = to_transport.base
 
         br_to = repository_to = dir_to = None
         try:
@@ -772,12 +771,12 @@
                 # Now we only need to create child directories
                 while needed:
                     cur_transport = needed.pop()
-                    cur_transport.mkdir('.')
-            
+                    cur_transport.ensure_base()
+
             # Now the target directory exists, but doesn't have a .bzr
             # directory. So we need to create it, along with any work to create
             # all of the dependent branches, etc.
-            dir_to = br_from.bzrdir.clone(location_url,
+            dir_to = br_from.bzrdir.clone_on_transport(to_transport,
                 revision_id=br_from.last_revision())
             br_to = dir_to.open_branch()
             # TODO: Some more useful message about what was copied
@@ -1284,11 +1283,8 @@
         # believe that we want to create a bunch of
         # locations if the user supplies an extended path
         # TODO: create-prefix
-        try:
-            to_transport.mkdir('.')
-        except errors.FileExists:
-            pass
-                    
+        to_transport.ensure_base()
+
         try:
             existing_bzrdir = bzrdir.BzrDir.open(location)
         except errors.NotBranchError:
@@ -1349,10 +1345,7 @@
             location = '.'
 
         to_transport = transport.get_transport(location)
-        try:
-            to_transport.mkdir('.')
-        except errors.FileExists:
-            pass
+        to_transport.ensure_base()
 
         newdir = format.initialize_on_transport(to_transport)
         repo = newdir.create_repository(shared=True)

=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2007-04-25 17:49:44 +0000
+++ bzrlib/bundle/bundle_data.py	2007-05-03 16:42:30 +0000
@@ -109,6 +109,9 @@
         self.timestamp = None
         self.timezone = None
 
+        # Have we checked the repository yet?
+        self._validated_revisions_against_repo = False
+
     def __str__(self):
         return pprint.pformat(self.__dict__)
 
@@ -180,7 +183,8 @@
         revision = self.get_revision(revision_id)
         base = self.get_base(revision)
         assert base != revision_id
-        self._validate_references_from_repository(repository)
+        if not self._validated_revisions_against_repo:
+            self._validate_references_from_repository(repository)
         revision_info = self.get_revision_info(revision_id)
         inventory_revision_id = revision_id
         bundle_tree = BundleTree(repository.revision_tree(base), 
@@ -262,6 +266,7 @@
             warning('Not all revision hashes could be validated.'
                     ' Unable validate %d hashes' % len(missing))
         mutter('Verified %d sha hashes for the bundle.' % count)
+        self._validated_revisions_against_repo = True
 
     def _validate_inventory(self, inv, revision_id):
         """At this point we should have generated the BundleTree,

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2007-04-26 08:15:40 +0000
+++ bzrlib/bzrdir.py	2007-05-02 14:54:20 +0000
@@ -155,8 +155,23 @@
         :param force_new_repo: Do not use a shared repository for the target 
                                even if one is available.
         """
-        self._make_tail(url)
-        result = self._format.initialize(url)
+        return self.clone_on_transport(get_transport(url),
+                                       revision_id=revision_id,
+                                       force_new_repo=force_new_repo)
+
+    def clone_on_transport(self, transport, revision_id=None,
+                           force_new_repo=False):
+        """Clone this bzrdir and its contents to transport verbatim.
+
+        If the target directory does not exist, it will be created.
+
+        if revision_id is not None, then the clone operation may tune
+            itself to download less data.
+        :param force_new_repo: Do not use a shared repository for the target 
+                               even if one is available.
+        """
+        transport.ensure_base()
+        result = self._format.initialize_on_transport(transport)
         try:
             local_repo = self.find_repository()
         except errors.NoRepositoryPresent:
@@ -195,13 +210,8 @@
     # TODO: This should be given a Transport, and should chdir up; otherwise
     # this will open a new connection.
     def _make_tail(self, url):
-        head, tail = urlutils.split(url)
-        if tail and tail != '.':
-            t = get_transport(head)
-            try:
-                t.mkdir(tail)
-            except errors.FileExists:
-                pass
+        t = get_transport(url)
+        t.ensure_base()
 
     # TODO: Should take a Transport
     @classmethod
@@ -217,13 +227,8 @@
         if cls is not BzrDir:
             raise AssertionError("BzrDir.create always creates the default"
                 " format, not one of %r" % cls)
-        head, tail = urlutils.split(base)
-        if tail and tail != '.':
-            t = get_transport(head)
-            try:
-                t.mkdir(tail)
-            except errors.FileExists:
-                pass
+        t = get_transport(base)
+        t.ensure_base()
         if format is None:
             format = BzrDirFormat.get_default_format()
         return format.initialize(safe_unicode(base))
@@ -756,9 +761,10 @@
         if revision_id is not None, then the clone operation may tune
             itself to download less data.
         """
-        self._make_tail(url)
+        target_transport = get_transport(url)
+        target_transport.ensure_base()
         cloning_format = self.cloning_metadir()
-        result = cloning_format.initialize(url)
+        result = cloning_format.initialize_on_transport(target_transport)
         try:
             source_branch = self.open_branch()
             source_repository = source_branch.repository

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2007-04-30 01:49:39 +0000
+++ bzrlib/msgeditor.py	2007-04-30 19:52:44 +0000
@@ -87,48 +87,33 @@
     This is run if they don't give a message or
     message-containing file on the command line.
 
-    infotext:
-        Text to be displayed at bottom of message for
-        the user's reference; currently similar to
-        'bzr status'.
-
-    ignoreline:
-        The separator to use above the infotext.
-
-    start_message:
-        The text to place above the separator, if any. This will not be
-        removed from the message after the user has edited it.
+    :param infotext:    Text to be displayed at bottom of message
+                        for the user's reference;
+                        currently similar to 'bzr status'.
+
+    :param ignoreline:  The separator to use above the infotext.
+
+    :param start_message:   The text to place above the separator, if any.
+                            This will not be removed from the message
+                            after the user has edited it.
+
+    :return:    commit message or None.
     """
-    import tempfile
-
     msgfilename = None
     try:
-        tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.',
-                                                   dir=u'.',
-                                                   text=True)
-        msgfile = os.fdopen(tmp_fileno, 'w')
-        try:
-            if start_message is not None:
-                msgfile.write("%s\n" % start_message.encode(
-                                           bzrlib.user_encoding, 'replace'))
-
-            if infotext is not None and infotext != "":
-                hasinfo = True
-                msgfile.write("\n\n%s\n\n%s" % (ignoreline,
-                              infotext.encode(bzrlib.user_encoding,
-                                                    'replace')))
-            else:
-                hasinfo = False
-        finally:
-            msgfile.close()
-
-        if not _run_editor(msgfilename):
+        msgfilename, hasinfo = _create_temp_file_with_commit_template(
+                                    infotext, ignoreline, start_message)
+
+        if not msgfilename or not _run_editor(msgfilename):
             return None
         
         started = False
         msg = []
         lastline, nlines = 0, 0
-        for line in codecs.open(msgfilename, 'r', bzrlib.user_encoding):
+        # codecs.open() ALWAYS opens file in binary mode but we need text mode
+        # 'rU' mode useful when bzr.exe used on Cygwin (bialix 20070430)
+        f = file(msgfilename, 'rU')
+        for line in codecs.getreader(bzrlib.user_encoding)(f):
             stripped_line = line.strip()
             # strip empty line before the log message starts
             if not started:
@@ -145,7 +130,8 @@
             if stripped_line != "":
                 lastline = nlines
             msg.append(line)
-            
+        f.close()
+
         if len(msg) == 0:
             return ""
         # delete empty lines at the end
@@ -164,6 +150,46 @@
                 warning("failed to unlink %s: %s; ignored", msgfilename, e)
 
 
+def _create_temp_file_with_commit_template(infotext,
+                                           ignoreline=DEFAULT_IGNORE_LINE,
+                                           start_message=None):
+    """Create temp file and write commit template in it.
+
+    :param infotext:    Text to be displayed at bottom of message
+                        for the user's reference;
+                        currently similar to 'bzr status'.
+
+    :param ignoreline:  The separator to use above the infotext.
+
+    :param start_message:   The text to place above the separator, if any.
+                            This will not be removed from the message
+                            after the user has edited it.
+
+    :return:    2-tuple (temp file name, hasinfo)
+    """
+    import tempfile
+    tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.',
+                                               dir=u'.',
+                                               text=True)
+    msgfile = os.fdopen(tmp_fileno, 'w')
+    try:
+        if start_message is not None:
+            msgfile.write("%s\n" % start_message.encode(
+                                       bzrlib.user_encoding, 'replace'))
+
+        if infotext is not None and infotext != "":
+            hasinfo = True
+            msgfile.write("\n\n%s\n\n%s" % (ignoreline,
+                          infotext.encode(bzrlib.user_encoding,
+                                                'replace')))
+        else:
+            hasinfo = False
+    finally:
+        msgfile.close()
+
+    return (msgfilename, hasinfo)
+
+
 def make_commit_message_template(working_tree, specific_files):
     """Prepare a template file for a commit into a branch.
 

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2007-04-26 09:07:38 +0000
+++ bzrlib/remote.py	2007-05-07 12:03:14 +0000
@@ -20,7 +20,7 @@
 from cStringIO import StringIO
 
 from bzrlib import branch, errors, lockdir, repository
-from bzrlib.branch import BranchReferenceFormat
+from bzrlib.branch import Branch, BranchReferenceFormat
 from bzrlib.bzrdir import BzrDir, RemoteBzrDirFormat
 from bzrlib.config import BranchConfig, TreeConfig
 from bzrlib.decorators import needs_read_lock, needs_write_lock
@@ -764,6 +764,11 @@
         self._lock_count = 0
         self._leave_lock = False
 
+    def __str__(self):
+        return "%s(%s)" % (self.__class__.__name__, self.base)
+
+    __repr__ = __str__
+
     def _ensure_real(self):
         """Ensure that there is a _real_branch set.
 
@@ -991,16 +996,24 @@
         return self._real_branch.append_revision(*revision_ids)
 
     @needs_write_lock
-    def pull(self, source, overwrite=False, stop_revision=None):
+    def pull(self, source, overwrite=False, stop_revision=None,
+             **kwargs):
+        # FIXME: This asks the real branch to run the hooks, which means
+        # they're called with the wrong target branch parameter. 
+        # The test suite specifically allows this at present but it should be
+        # fixed.  It should get a _override_hook_target branch,
+        # as push does.  -- mbp 20070405
         self._ensure_real()
         self._real_branch.pull(
-            source, overwrite=overwrite, stop_revision=stop_revision)
+            source, overwrite=overwrite, stop_revision=stop_revision,
+            **kwargs)
 
     @needs_read_lock
     def push(self, target, overwrite=False, stop_revision=None):
         self._ensure_real()
         return self._real_branch.push(
-            target, overwrite=overwrite, stop_revision=stop_revision)
+            target, overwrite=overwrite, stop_revision=stop_revision,
+            _override_hook_source_branch=self)
 
     def is_locked(self):
         return self._lock_count >= 1

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2007-05-01 16:11:36 +0000
+++ bzrlib/symbol_versioning.py	2007-05-14 17:21:02 +0000
@@ -35,6 +35,7 @@
            'zero_fourteen',
            'zero_fifteen',
            'zero_sixteen',
+           'zero_seventeen',
            ]
 
 from warnings import warn
@@ -51,6 +52,7 @@
 zero_fourteen = "%s was deprecated in version 0.14."
 zero_fifteen = "%s was deprecated in version 0.15."
 zero_sixteen = "%s was deprecated in version 0.16."
+zero_seventeen = "%s was deprecated in version 0.17."
 
 
 def set_warning_method(method):

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-05-01 16:11:36 +0000
+++ bzrlib/tests/__init__.py	2007-05-14 17:21:02 +0000
@@ -823,7 +823,7 @@
             message += '\n'
         raise AssertionError("%snot equal:\na = %s\nb = %s\n"
             % (message,
-               pformat(a, indent=4), pformat(b, indent=4)))
+               pformat(a), pformat(b)))
 
     assertEquals = assertEqual
 
@@ -1805,10 +1805,7 @@
             segments = maybe_a_url.rsplit('/', 1)
             t = get_transport(maybe_a_url)
             if len(segments) > 1 and segments[-1] not in ('', '.'):
-                try:
-                    t.mkdir('.')
-                except errors.FileExists:
-                    pass
+                t.ensure_base()
             if format is None:
                 format = 'default'
             if isinstance(format, basestring):
@@ -1964,8 +1961,7 @@
     def assertFileEqual(self, content, path):
         """Fail if path does not contain 'content'."""
         self.failUnlessExists(path)
-        # TODO: jam 20060427 Shouldn't this be 'rb'?
-        f = file(path, 'r')
+        f = file(path, 'rb')
         try:
             s = f.read()
         finally:

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2007-04-25 06:50:22 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2007-05-04 10:13:17 +0000
@@ -559,6 +559,10 @@
         out_all,err_all = self.run_bzr_captured(['selftest', '--list-only',
             'selftest'])
         tests_all = self._parse_test_list(out_all.splitlines())[1]
+        # XXX: It looks like there are some orders for generating tests that
+        # fail as of 20070504 - maybe because of import order dependencies.
+        # So unfortunately this will rarely intermittently fail at the moment.
+        # -- mbp 20070504
         out_rand,err_rand = self.run_bzr_captured(['selftest', '--list-only',
             'selftest', '--randomize', 'now'])
         (header_rand,tests_rand,dummy) = self._parse_test_list(

=== modified file 'bzrlib/tests/branch_implementations/test_pull.py'
--- bzrlib/tests/branch_implementations/test_pull.py	2007-03-01 07:15:55 +0000
+++ bzrlib/tests/branch_implementations/test_pull.py	2007-05-04 08:46:39 +0000
@@ -18,10 +18,12 @@
 
 import os
 
-from bzrlib.branch import Branch
+from bzrlib.branch import Branch, BzrBranchFormat5
+from bzrlib.bzrdir import BzrDir
 from bzrlib import errors
 from bzrlib.memorytree import MemoryTree
 from bzrlib.revision import NULL_REVISION
+from bzrlib.tests import TestSkipped
 from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
 
 
@@ -142,8 +144,13 @@
         try:
             local.bind(target)
         except errors.UpgradeRequired:
-            # cant bind this format, the test is irrelevant.
-            return
+            # We can't bind this format to itself- typically it is the local
+            # branch that doesn't support binding.  As of May 2007
+            # remotebranches can't be bound.  Let's instead make a new local
+            # branch of the default type, which does allow binding.
+            # See https://bugs.launchpad.net/bzr/+bug/112020
+            local = BzrDir.create_branch_convenience('local2')
+            local.bind(target)
         source = self.make_branch('source')
         Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
         local.pull(source)

=== modified file 'bzrlib/tests/branch_implementations/test_push.py'
--- bzrlib/tests/branch_implementations/test_push.py	2007-03-29 07:39:49 +0000
+++ bzrlib/tests/branch_implementations/test_push.py	2007-05-07 12:03:14 +0000
@@ -17,12 +17,14 @@
 """Tests for branch.push behaviour."""
 
 import os
-
+ 
 from bzrlib import bzrdir, errors
 from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
 from bzrlib.memorytree import MemoryTree
 from bzrlib.remote import RemoteBranch
 from bzrlib.revision import NULL_REVISION
+from bzrlib.tests import TestSkipped
 from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
 from bzrlib.transport.local import LocalURLServer
 
@@ -185,12 +187,6 @@
         source.push(target)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.
-        if isinstance(source, RemoteBranch):
-            # XXX: at the moment, push on remote branches is just delegated to
-            # the file-level branch object, so we adjust the expected result
-            # accordingly.  In the future, when RemoteBranch implements push
-            # directly, this should be unnecessary.
-            source = source._real_branch
         self.assertEqual([
             ('post_push', source, None, target.base, 0, NULL_REVISION,
              0, NULL_REVISION, True, None, True)
@@ -207,8 +203,13 @@
         try:
             local.bind(target)
         except errors.UpgradeRequired:
-            # cant bind this format, the test is irrelevant.
-            return
+            # We can't bind this format to itself- typically it is the local
+            # branch that doesn't support binding.  As of May 2007
+            # remotebranches can't be bound.  Let's instead make a new local
+            # branch of the default type, which does allow binding.
+            # See https://bugs.launchpad.net/bzr/+bug/112020
+            local = BzrDir.create_branch_convenience('local2')
+            local.bind(target)
         source = self.make_branch('source')
         Branch.hooks.install_hook('post_push', self.capture_post_push_hook)
         source.push(local)

=== modified file 'bzrlib/tests/bzrdir_implementations/__init__.py'
--- bzrlib/tests/bzrdir_implementations/__init__.py	2007-04-17 09:16:29 +0000
+++ bzrlib/tests/bzrdir_implementations/__init__.py	2007-05-01 22:41:41 +0000
@@ -28,12 +28,29 @@
 from bzrlib.tests import (
                           adapt_modules,
                           default_transport,
+                          TestCaseWithTransport,
                           TestLoader,
                           TestSuite,
                           )
 from bzrlib.transport.memory import MemoryServer
 
 
+class TestCaseWithBzrDir(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestCaseWithBzrDir, self).setUp()
+        self.bzrdir = None
+
+    def get_bzrdir(self):
+        if self.bzrdir is None:
+            self.bzrdir = self.make_bzrdir(None)
+        return self.bzrdir
+
+    def make_bzrdir(self, relpath, format=None):
+        return super(TestCaseWithBzrDir, self).make_bzrdir(
+            relpath, format=self.bzrdir_format)
+
+
 def test_suite():
     result = TestSuite()
     test_bzrdir_implementations = [

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-04-23 08:30:30 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-05-01 22:41:41 +0000
@@ -48,6 +48,7 @@
                           TestCaseWithTransport,
                           TestSkipped,
                           )
+from bzrlib.tests.bzrdir_implementations import TestCaseWithBzrDir
 from bzrlib.trace import mutter
 from bzrlib.transport import get_transport
 from bzrlib.upgrade import upgrade
@@ -55,23 +56,6 @@
 from bzrlib.repofmt import weaverepo
 
 
-class TestCaseWithBzrDir(TestCaseWithTransport):
-
-    def setUp(self):
-        super(TestCaseWithBzrDir, self).setUp()
-        self.bzrdir = None
-
-    def get_bzrdir(self):
-        if self.bzrdir is None:
-            self.bzrdir = self.make_bzrdir(None)
-        return self.bzrdir
-
-    def make_bzrdir(self, relpath, format=None):
-        return super(TestCaseWithBzrDir, self).make_bzrdir(
-            relpath, format=self.bzrdir_format)
-
-
-
 class TestBzrDir(TestCaseWithBzrDir):
     # Many of these tests test for disk equality rather than checking
     # for semantic equivalence. This works well for some tests but
@@ -191,7 +175,15 @@
             # so this test is irrelevant.
             return
         self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
-            
+
+    def test_clone_on_transport(self):
+        a_dir = self.make_bzrdir('source')
+        target_transport = a_dir.root_transport.clone('..').clone('target')
+        target = a_dir.clone_on_transport(target_transport)
+        self.assertNotEqual(a_dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(a_dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
+
     def test_clone_bzrdir_empty(self):
         dir = self.make_bzrdir('source')
         target = dir.clone(self.get_url('target'))

=== modified file 'bzrlib/tests/intertree_implementations/test_compare.py'
--- bzrlib/tests/intertree_implementations/test_compare.py	2007-04-26 20:45:53 +0000
+++ bzrlib/tests/intertree_implementations/test_compare.py	2007-04-30 17:45:10 +0000
@@ -867,6 +867,51 @@
             specific_files=specific_files, require_versioned=False,
             want_unversioned=True))
 
+    def test_similar_filenames(self):
+        """Test when we have a few files with similar names."""
+        tree1 = self.make_branch_and_tree('tree1')
+        tree2 = self.make_branch_and_tree('tree2')
+        tree2.set_root_id(tree1.get_root_id())
+
+        # The trees are actually identical, but they happen to contain
+        # similarly named files.
+        self.build_tree(['tree1/a/',
+                         'tree1/a/b/',
+                         'tree1/a/b/c/',
+                         'tree1/a/b/c/d/',
+                         'tree1/a-c/',
+                         'tree1/a-c/e/',
+                         'tree2/a/',
+                         'tree2/a/b/',
+                         'tree2/a/b/c/',
+                         'tree2/a/b/c/d/',
+                         'tree2/a-c/',
+                         'tree2/a-c/e/',
+                        ])
+        tree1.add(['a', 'a/b', 'a/b/c', 'a/b/c/d', 'a-c', 'a-c/e'],
+                  ['a-id', 'b-id', 'c-id', 'd-id', 'a-c-id', 'e-id'])
+        tree2.add(['a', 'a/b', 'a/b/c', 'a/b/c/d', 'a-c', 'a-c/e'],
+                  ['a-id', 'b-id', 'c-id', 'd-id', 'a-c-id', 'e-id'])
+
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+
+        self.assertEqual([], self.do_iter_changes(tree1, tree2,
+                                                  want_unversioned=True))
+        expected = sorted([
+            self.unchanged(tree2, tree2.get_root_id()),
+            self.unchanged(tree2, 'a-id'),
+            self.unchanged(tree2, 'b-id'),
+            self.unchanged(tree2, 'c-id'),
+            self.unchanged(tree2, 'd-id'),
+            self.unchanged(tree2, 'a-c-id'),
+            self.unchanged(tree2, 'e-id'),
+            ])
+        self.assertEqual(expected,
+                         self.do_iter_changes(tree1, tree2,
+                                              want_unversioned=True,
+                                              include_unchanged=True))
+
+
     def test_unversioned_subtree_only_emits_root(self):
         tree1 = self.make_branch_and_tree('tree1')
         tree2 = self.make_to_branch_and_tree('tree2')
@@ -1354,3 +1399,34 @@
         self.assertEqual(expected,
                          self.do_iter_changes(tree1, tree2,
                                               want_unversioned=True))
+
+    def test_renamed_and_unknown(self):
+        """A file was moved on the filesystem, but not in bzr."""
+        tree1 = self.make_branch_and_tree('tree1')
+        tree2 = self.make_to_branch_and_tree('tree2')
+        root_id = tree1.get_root_id()
+        tree2.set_root_id(root_id)
+
+        # The final changes are:
+        # bzr add a b
+        # mv a a2
+
+        self.build_tree_contents([
+            ('tree1/a', 'a contents\n'),
+            ('tree1/b', 'b contents\n'),
+            ('tree2/a', 'a contents\n'),
+            ('tree2/b', 'b contents\n'),
+            ])
+        tree1.add(['a', 'b'], ['a-id', 'b-id'])
+        tree2.add(['a', 'b'], ['a-id', 'b-id'])
+        os.rename('tree2/a', 'tree2/a2')
+
+        tree1, tree2 = self.mutable_trees_to_locked_test_trees(tree1, tree2)
+
+        expected = sorted([
+            self.missing('a-id', 'a', 'a', tree2.get_root_id(), 'file'),
+            self.unversioned(tree2, 'a2'),
+            ])
+        self.assertEqual(expected,
+                         self.do_iter_changes(tree1, tree2,
+                                              want_unversioned=True))

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2007-02-04 21:58:50 +0000
+++ bzrlib/tests/test_msgeditor.py	2007-05-01 09:09:18 +0000
@@ -187,3 +187,26 @@
                 del os.environ['EDITOR']
             else:
                 os.environ['EDITOR'] = editor
+
+    def test__create_temp_file_with_commit_template(self):
+        # check that commit template written properly
+        # and has platform native line-endings (CRLF on win32)
+        create_file = bzrlib.msgeditor._create_temp_file_with_commit_template
+        msgfilename, hasinfo = create_file('infotext','----','start message')
+        self.assertNotEqual(None, msgfilename)
+        self.assertTrue(hasinfo)
+        expected = os.linesep.join(['start message',
+                                    '',
+                                    '',
+                                    '----',
+                                    '',
+                                    'infotext'])
+        self.assertFileEqual(expected, msgfilename)
+
+    def test__create_temp_file_with_empty_commit_template(self):
+        # empty file
+        create_file = bzrlib.msgeditor._create_temp_file_with_commit_template
+        msgfilename, hasinfo = create_file('')
+        self.assertNotEqual(None, msgfilename)
+        self.assertFalse(hasinfo)
+        self.assertFileEqual('', msgfilename)

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2007-04-26 05:04:40 +0000
+++ bzrlib/tests/test_plugins.py	2007-05-02 12:00:36 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -250,19 +250,21 @@
         index = plugin.PluginsHelpIndex()
         self.assertEqual([], index.get_topics(None))
 
-    def test_get_topics_launchpad(self):
-        """Searching for 'launchpad' returns the launchpad plugin docstring."""
+    def test_get_topics_for_plugin(self):
+        """Searching for plugin name gets its docstring."""
         index = plugin.PluginsHelpIndex()
-        self.assertFalse(sys.modules.has_key('bzrlib.plugins.get_topics'))
-        demo_module = FakeModule('', 'bzrlib.plugins.get_topics')
-        sys.modules['bzrlib.plugins.get_topics'] = demo_module
+        # make a new plugin here for this test, even if we're run with
+        # --no-plugins
+        self.assertFalse(sys.modules.has_key('bzrlib.plugins.demo_module'))
+        demo_module = FakeModule('', 'bzrlib.plugins.demo_module')
+        sys.modules['bzrlib.plugins.demo_module'] = demo_module
         try:
-            topics = index.get_topics('get_topics')
+            topics = index.get_topics('demo_module')
             self.assertEqual(1, len(topics))
             self.assertIsInstance(topics[0], plugin.ModuleHelpTopic)
             self.assertEqual(demo_module, topics[0].module)
         finally:
-            del sys.modules['bzrlib.plugins.get_topics']
+            del sys.modules['bzrlib.plugins.demo_module']
 
     def test_get_topics_no_topic(self):
         """Searching for something that is not a plugin returns []."""
@@ -276,19 +278,19 @@
         index = plugin.PluginsHelpIndex()
         self.assertEqual('plugins/', index.prefix)
 
-    def test_get_topic_with_prefix(self):
-        """Searching for plugins/launchpad returns launchpad module help."""
+    def test_get_plugin_topic_with_prefix(self):
+        """Searching for plugins/demo_module returns help."""
         index = plugin.PluginsHelpIndex()
-        self.assertFalse(sys.modules.has_key('bzrlib.plugins.get_topics'))
-        demo_module = FakeModule('', 'bzrlib.plugins.get_topics')
-        sys.modules['bzrlib.plugins.get_topics'] = demo_module
+        self.assertFalse(sys.modules.has_key('bzrlib.plugins.demo_module'))
+        demo_module = FakeModule('', 'bzrlib.plugins.demo_module')
+        sys.modules['bzrlib.plugins.demo_module'] = demo_module
         try:
-            topics = index.get_topics('plugins/get_topics')
+            topics = index.get_topics('plugins/demo_module')
             self.assertEqual(1, len(topics))
             self.assertIsInstance(topics[0], plugin.ModuleHelpTopic)
             self.assertEqual(demo_module, topics[0].module)
         finally:
-            del sys.modules['bzrlib.plugins.get_topics']
+            del sys.modules['bzrlib.plugins.demo_module']
 
 
 class FakeModule(object):

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2007-04-30 02:04:06 +0000
+++ bzrlib/tests/test_remote.py	2007-05-07 13:11:40 +0000
@@ -98,6 +98,10 @@
         d = fmt.open(self.transport)
         self.assertIsInstance(d, BzrDir)
 
+    def test_remote_branch_repr(self):
+        b = BzrDir.open_from_transport(self.transport).open_branch()
+        self.assertStartsWith(str(b), 'RemoteBranch(')
+
 
 class FakeProtocol(object):
     """Lookalike SmartClientRequestProtocolOne allowing body reading tests."""
@@ -422,6 +426,20 @@
             [('call', 'Transport.is_readonly', ())],
             client._calls)
 
+    def test_error_from_old_0_11_server(self):
+        """Same as test_error_from_old_server, but with the slightly different
+        error message from bzr 0.11 servers.
+        """
+        client = FakeClient([(
+            ('error', "Generic bzr smart protocol error: "
+                      "bad request u'Transport.is_readonly'"), '')])
+        transport = RemoteTransport('bzr://example.com/', medium=False,
+                                    _client=client)
+        self.assertEqual(False, transport.is_readonly())
+        self.assertEqual(
+            [('call', 'Transport.is_readonly', ())],
+            client._calls)
+
 
 class TestRemoteRepository(tests.TestCase):
     """Base for testing RemoteRepository protocol usage.

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2007-04-26 09:07:38 +0000
+++ bzrlib/tests/test_transport.py	2007-05-04 10:41:31 +0000
@@ -592,3 +592,13 @@
         # make sure we reach the root
         t = t.clone('..')
         self.assertEquals(t.base, 'file://HOST/')
+
+
+def get_test_permutations():
+    """Return transport permutations to be used in testing.
+
+    This module registers some transports, but they're only for testing
+    registration.  We don't really want to run all the transport tests against
+    them.
+    """
+    return []

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2007-04-23 05:03:44 +0000
+++ bzrlib/tests/test_transport_implementations.py	2007-05-02 14:36:55 +0000
@@ -61,6 +61,38 @@
         """Check that transport.get(relpath).read() == content."""
         self.assertEqualDiff(content, transport.get(relpath).read())
 
+    def test_ensure_base_missing(self):
+        """.ensure_base() should create the directory if it doesn't exist"""
+        t = self.get_transport()
+        t_a = t.clone('a')
+        if t_a.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                              t_a.ensure_base)
+            return
+        self.assertTrue(t_a.ensure_base())
+        self.assertTrue(t.has('a'))
+
+    def test_ensure_base_exists(self):
+        """.ensure_base() should just be happy if it already exists"""
+        t = self.get_transport()
+        if t.is_readonly():
+            return
+
+        t.mkdir('a')
+        t_a = t.clone('a')
+        # ensure_base returns False if it didn't create the base
+        self.assertFalse(t_a.ensure_base())
+
+    def test_ensure_base_missing_parent(self):
+        """.ensure_base() will fail if the parent dir doesn't exist"""
+        t = self.get_transport()
+        if t.is_readonly():
+            return
+
+        t_a = t.clone('a')
+        t_b = t_a.clone('b')
+        self.assertRaises(NoSuchFile, t_b.ensure_base)
+
     def test_has(self):
         t = self.get_transport()
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2007-04-26 09:07:38 +0000
+++ bzrlib/transport/__init__.py	2007-05-08 20:00:50 +0000
@@ -276,6 +276,22 @@
         """
         raise NotImplementedError(self.clone)
 
+    def ensure_base(self):
+        """Ensure that the directory this transport references exists.
+
+        This will create a directory if it doesn't exist.
+        :return: True if the directory was created, False otherwise.
+        """
+        # The default implementation just uses "Easier to ask for forgiveness
+        # than permission". We attempt to create the directory, and just
+        # suppress a FileExists exception.
+        try:
+            self.mkdir('.')
+        except errors.FileExists:
+            return False
+        else:
+            return True
+
     def should_cache(self):
         """Return True if the data pulled across should be cached locally.
         """
@@ -1185,8 +1201,10 @@
     def get_transport_test_permutations(self, module):
         """Get the permutations module wants to have tested."""
         if getattr(module, 'get_test_permutations', None) is None:
-            warning("transport module %s doesn't provide get_test_permutations()"
+            raise AssertionError("transport module %s doesn't provide get_test_permutations()"
                     % module.__name__)
+            ##warning("transport module %s doesn't provide get_test_permutations()"
+            ##       % module.__name__)
             return []
         return module.get_test_permutations()
 

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2007-04-30 04:00:06 +0000
+++ bzrlib/transport/remote.py	2007-04-30 06:42:24 +0000
@@ -133,8 +133,10 @@
             return True
         elif resp == ('no', ):
             return False
-        elif resp == ('error', "Generic bzr smart protocol error: "
-                               "bad request 'Transport.is_readonly'"):
+        elif (resp == ('error', "Generic bzr smart protocol error: "
+                                "bad request 'Transport.is_readonly'") or
+              resp == ('error', "Generic bzr smart protocol error: "
+                                "bad request u'Transport.is_readonly'")):
             # XXX: nasty hack: servers before 0.16 don't have a
             # 'Transport.is_readonly' verb, so we do what clients before 0.16
             # did: assume False.

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-04-25 06:50:22 +0000
+++ bzrlib/workingtree.py	2007-05-02 14:54:20 +0000
@@ -932,20 +932,14 @@
             transport = self.branch.bzrdir.root_transport
             for name in segments:
                 transport = transport.clone(name)
-                try:
-                    transport.mkdir('.')
-                except errors.FileExists:
-                    pass
+                transport.ensure_base()
             return transport
             
         sub_path = self.id2path(file_id)
         branch_transport = mkdirs(sub_path)
         if format is None:
             format = bzrdir.format_registry.make_bzrdir('dirstate-with-subtree')
-        try:
-            branch_transport.mkdir('.')
-        except errors.FileExists:
-            pass
+        branch_transport.ensure_base()
         branch_bzrdir = format.initialize_on_transport(branch_transport)
         try:
             repo = branch_bzrdir.find_repository()

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2007-04-30 05:13:58 +0000
+++ bzrlib/workingtree_4.py	2007-04-30 17:45:10 +0000
@@ -2122,7 +2122,7 @@
                    current_block is not None):
                 if (current_dir_info and current_block
                     and current_dir_info[0][0] != current_block[0]):
-                    if current_dir_info[0][0] < current_block[0] :
+                    if current_dir_info[0][0].split('/') < current_block[0].split('/'):
                         # filesystem data refers to paths not covered by the dirblock.
                         # this has two possibilities:
                         # A) it is versioned but empty, so there is no block for it
@@ -2256,8 +2256,14 @@
                                        result[6],
                                        result[7],
                                       )
-                    elif current_entry[0][1] != current_path_info[1]:
-                        if current_path_info[1] < current_entry[0][1]:
+                    elif (current_entry[0][1] != current_path_info[1]
+                          or current_entry[1][target_index][0] in 'ar'):
+                        # The current path on disk doesn't match the dirblock
+                        # record. Either the dirblock is marked as absent, or
+                        # the file on disk is not present at all in the
+                        # dirblock. Either way, report about the dirblock
+                        # entry, and let other code handle the filesystem one.
+                        if current_path_info[1].split('/') < current_entry[0][1].split('/'):
                             # extra file on disk: pass for now, but only
                             # increment the path, not the entry
                             advance_entry = False
@@ -2268,7 +2274,6 @@
                                 # this check should probably be outside the loop: one
                                 # 'iterate two trees' api, and then _iter_changes filters
                                 # unchanged pairs. - RBC 20070226
-                                path_handled = True
                                 if (include_unchanged
                                     or result[2]                    # content change
                                     or result[3][0] != result[3][1] # versioned status
@@ -2289,33 +2294,6 @@
                                            result[7],
                                           )
                             advance_path = False
-                    elif current_entry[1][target_index][0] in 'ar':
-                        # The path matches, but the current entry is marked as
-                        # not being here. So we don't want to consider this
-                        # as a match. We still need to process the current
-                        # entry, though.
-                        advance_path = False
-                        path_handled = False
-                        for result in _process_entry(current_entry, None):
-                            if (include_unchanged
-                                or result[2]                    # content change
-                                or result[3][0] != result[3][1] # versioned status
-                                or result[4][0] != result[4][1] # parent id
-                                or result[5][0] != result[5][1] # name
-                                or result[6][0] != result[6][1] # kind
-                                or result[7][0] != result[7][1] # executable
-                                ):
-                                yield (result[0],
-                                       (utf8_decode_or_none(result[1][0]),
-                                        utf8_decode_or_none(result[1][1])),
-                                       result[2],
-                                       result[3],
-                                       result[4],
-                                       (utf8_decode_or_none(result[5][0]),
-                                        utf8_decode_or_none(result[5][1])),
-                                       result[6],
-                                       result[7],
-                                      )
                     else:
                         for result in _process_entry(current_entry, current_path_info):
                             # this check should probably be outside the loop: one

=== modified file 'doc/centralized_workflow.txt'
--- doc/centralized_workflow.txt	2007-03-02 12:40:45 +0000
+++ doc/centralized_workflow.txt	2007-05-07 16:55:17 +0000
@@ -235,7 +235,7 @@
 
    % bzr cbranch dev my-feature-branch
 
-.. _bzrtools: https://launchpad.net/products/bzrtools
+.. _bzrtools: http://bazaar-vcs.org/BzrTools
 
 
 Merging changes back

=== modified file 'doc/default.css'
--- doc/default.css	2006-08-29 03:21:56 +0000
+++ doc/default.css	2007-05-03 06:21:49 +0000
@@ -9,7 +9,7 @@
   margin-bottom: 70px;
   font-family: Verdana, Geneva, Arial, sans-serif;
   font-size: small;
-  line-height: 180%
+  line-height: 140%
   }
 
 /* p {
@@ -111,6 +111,7 @@
 
 tt, .literal-block {
   font-family: monospace;
+  line-height: 100%
   }
 
 tt {
@@ -120,6 +121,7 @@
 
 .literal-block {
   margin-left: 5em;
+  margin-right: 5em;
   color: #000000;
   font-weight: normal;
   background-color: #c1d1ff;

=== added directory 'doc/developers'
=== renamed file 'HACKING' => 'doc/developers/HACKING'
--- HACKING	2007-04-24 14:19:24 +0000
+++ doc/developers/HACKING	2007-05-07 16:48:14 +0000
@@ -1,14 +1,227 @@
-============================
-Guidelines for modifying bzr
-============================
+======================
+Bazaar Developer Guide
+======================
 
 .. contents::
 
 (The current version of this document is available in the file ``HACKING``
-in the source tree, or at http://doc.bazaar-vcs.org/current/hacking.html)
-
-Overall
-=======
+in the source tree, or at http://doc.bazaar-vcs.org/bzr.dev/hacking.html)
+
+
+Getting Started
+###############
+
+Exploring the Bazaar Platform
+=============================
+
+Before making changes, it's a good idea to explore the work already
+done by others. Perhaps the new feature or improvement you're looking
+for is available in another plug-in already? If you find a bug,
+perhaps someone else has already fixed it?
+
+To answer these questions and more, take a moment to explore the
+overall Bazaar Platform. Here are some links to browse:
+
+* The Plugins page on the Wiki - http://bazaar-vcs.org/BzrPlugins
+
+* The Bazaar product family on Launchpad - https://launchpad.net/bazaar
+
+* Bug Tracker for the core product - https://bugs.launchpad.net/bzr/
+
+* Blueprint Tracker for the core product - https://blueprints.launchpad.net/bzr/
+
+If nothing else, perhaps you'll find inspiration in how other developers
+have solved their challenges.
+
+
+Planning and Discussing Changes
+===============================
+
+There is a very active community around Bazaar. Mostly we meet on IRC
+(#bzr on irc.freenode.net) and on the mailing list. To join the Bazaar
+community, see http://bazaar-vcs.org/BzrSupport.
+
+If you are planning to make a change, it's a very good idea to mention it
+on the IRC channel and/or on the mailing list. There are many advantages
+to involving the community before you spend much time on a change.
+These include:
+
+* you get to build on the wisdom on others, saving time
+
+* if others can direct you to similar code, it minimises the work to be done 
+
+* it assists everyone in coordinating direction, priorities and effort.
+
+In summary, maximising the input from others typically minimises the
+total effort required to get your changes merged. The community is
+friendly, helpful and always keen to welcome newcomers.
+
+
+Bazaar Development in a Nutshell
+================================
+
+Looking for a 10 minute introduction to submitting a change?
+See http://bazaar-vcs.org/BzrGivingBack.
+
+TODO: Merge that Wiki page into this document.
+
+
+Understanding the Development Process
+=====================================
+
+The development team follows many best-practices including:
+
+* a public roadmap and planning process in which anyone can participate
+
+* time based milestones everyone can work towards and plan around
+
+* extensive code review and feedback to contributors
+
+* complete and rigorous test coverage on any code contributed
+
+* automated validation that all tests still pass before code is merged
+  into the main code branch.
+
+The key tools we use to enable these practices are:
+
+* Launchpad - https://launchpad.net/
+
+* Bazaar - http://bazaar-vcs.org/
+
+* Bundle Buggy - http://bundlebuggy.aaronbentley.com/
+
+* Patch Queue Manager - https://launchpad.net/pqm/
+
+For further information, see http://bazaar-vcs.org/BzrDevelopment.
+
+
+A Closer Look at the Merge & Review Process
+===========================================
+
+If you'd like to propose a change, please post to the
+bazaar@lists.canonical.com list with a bundle, patch, or link to a
+branch. Put '[PATCH]' or '[MERGE]' in the subject so Bundle Buggy
+can pick it out, and explain the change in the email message text.
+Remember to update the NEWS file as part of your change if it makes any
+changes visible to users or plugin developers. Please include a diff
+against mainline if you're giving a link to a branch.
+
+You can generate a bundle like this::
+
+  bzr bundle > mybundle.patch
+  
+A .patch extension is recommended instead of .bundle as many mail clients
+will send the latter as a binary file. If a bundle would be too long or your
+mailer mangles whitespace (e.g. implicitly converts Unix newlines to DOS
+newlines), use the merge-directive command instead like this::
+
+  bzr merge-directive http://bazaar-vcs.org http://example.org/my_branch > my_directive.patch
+
+See the help for details on the arguments to merge-directive.
+
+Please do **NOT** put [PATCH] or [MERGE] in the subject line if you don't
+want it to be merged. If you want comments from developers rather than
+to be merged, you can put '[RFC]' in the subject line.
+
+Anyone is welcome to review code.  There are broadly three gates for
+code to get in:
+
+ * Doesn't reduce test coverage: if it adds new methods or commands,
+   there should be tests for them.  There is a good test framework
+   and plenty of examples to crib from, but if you are having trouble
+   working out how to test something feel free to post a draft patch
+   and ask for help.
+
+ * Doesn't reduce design clarity, such as by entangling objects
+   we're trying to separate.  This is mostly something the more
+   experienced reviewers need to help check.
+
+ * Improves bugs, features, speed, or code simplicity.
+
+Code that goes in should pass all three. The core developers take care
+to keep the code quality high and understandable while recognising that
+perfect is sometimes the enemy of good. (It is easy for reviews to make
+people notice other things which should be fixed but those things should
+not hold up the original fix being accepted. New things can easily be
+recorded in the Bug Tracker instead.)
+
+Anyone can "vote" on the mailing list. Core developers can also vote using
+Bundle Buggy. Here are the voting codes and their explanations.
+
+  -1	really don't want it in current form
+  -0	somewhat uncomfortable 
+  +0	comfortable but resubmission after changes requested
+  +1 conditional	good to go after some minor changes
+  +1	good to go
+
++1 conditional is used as a way to avoid another submit/review cycle for
+patches that need small changes.
+
+If a change gets two +1 votes from core reviewers, and no
+vetos, then it's OK to come in.  Any of the core developers can bring it
+into the bzr.dev trunk and backport it to maintenance branches if required.
+The Release Manager will merge the change into the branch for a pending
+release, if any. As a guideline, core developers usually merge their own
+changes and volunteer to merge other contributions if they were the second
+reviewer to agree to a change.
+
+To track the progress of proposed changes, use Bundle Buggy. See
+http://bundlebuggy.aaronbentley.com/help for a link to all the
+outstanding merge requests together with an explanation of the columns.
+Bundle Buggy will also mail you a link to track just your change.
+
+
+Preparing a Sandbox for Making Changes to Bazaar
+================================================
+
+Bazaar supports many ways of organising your work. See
+http://bazaar-vcs.org/SharedRepositoryLayouts for a summary of the
+popular alternatives.
+
+Of course, the best choice for you will depend on numerous factors:
+the number of changes you may be making, the complexity of the changes, etc.
+As a starting suggestion though:
+
+* create a local copy of the main development branch (bzr.dev) by using
+  this command::
+  
+    bzr branch http://bazaar-vcs.org/bzr/bzr.dev/ bzr.dev
+   
+* keep your copy of bzr.dev prestine (by not developing in it) and keep
+  it up to date (by using bzr pull)
+
+* create a new branch off your local bzr.dev copy for each issue
+  (bug or feature) you are working on.
+
+This approach makes it easy to go back and make any required changes
+after a code review. Resubmitting the change is then simple with no
+risk of accidentially including edits related to other issues you may
+be working on. After the changes for an issue are accepted and merged,
+the associated branch can be deleted or archived as you wish.
+
+
+Navigating the Code Base
+========================
+
+TODO: List and describe in one line the purpose of each directory
+inside an installation of bzr.
+
+TODO: Refer to a central location holding an up to date copy of the API
+documentation generated by epydoc, e.g. something like
+http://starship.python.net/crew/mwh/bzrlibapi/bzrlib.html.
+
+
+Testing Bazaar
+##############
+
+The Importance of Testing
+=========================
+
+Reliability is a critical success factor for any Version Control System.
+We want Bazaar to be highly reliable across multiple platforms while
+evolving over time to meet the needs of its community. 
+
+In a nutshell, this is want we expect and encourage:
 
 * New functionality should have test cases.  Preferably write the
   test before writing the code.
@@ -16,37 +229,224 @@
   In general, you can test at either the command-line level or the
   internal API level.  See Writing Tests below for more detail.
 
-* Try to practice Test-Driven Development.  before fixing a bug, write a
+* Try to practice Test-Driven Development: before fixing a bug, write a
   test case so that it does not regress.  Similarly for adding a new
   feature: write a test case for a small version of the new feature before
   starting on the code itself.  Check the test fails on the old code, then
   add the feature or fix and check it passes.
 
-* Exceptions should be defined inside bzrlib.errors, so that we can
-  see the whole tree at a glance.
-
-* Imports should be done at the top-level of the file, unless there is
-  a strong reason to have them lazily loaded when a particular
-  function runs.  Import statements have a cost, so try to make sure
-  they don't run inside hot functions.
-
-* Module names should always be given fully-qualified,
-  i.e. ``bzrlib.hashcache`` not just ``hashcache``.
-
-* Commands should return non-zero when they encounter circumstances that
-  the user should really pay attention to - which includes trivial shell
-  pipelines.
-
-  Recommended values are 
-    0. OK, 
-    1. Conflicts in merge-like operations, or changes are present in
-       diff-like operations. 
-    2. Unrepresentable diff changes (i.e. binary files that we cannot show 
-       a diff of).
-    3. An error or exception has occurred.
-
-Evolving interfaces
--------------------
+By doing these things, the Bazaar team gets increased confidence that
+changes do what they claim to do, whether provided by the core team or
+by community members. Equally importantly, we can be surer that changes
+down the track do not break new features or bug fixes that you are
+contributing today.
+
+As of May 2007, Bazaar ships with a test suite containing over 6000 tests
+and growing. We are proud of it and want to remain so. As community
+members, we all benefit from it. Would you trust version control on
+your project to a product *without* a test suite like Bazaar has?
+
+
+Running the Test Suite
+======================
+
+Currently, bzr selftest is used to invoke tests.
+You can provide a pattern argument to run a subset. For example, 
+to run just the blackbox tests, run::
+
+  ./bzr selftest -v blackbox
+
+To skip a particular test (or set of tests), use the --exclude option
+(shorthand -x) like so::
+
+  ./bzr selftest -v -x blackbox  
+
+To list tests without running them, use the --list-only option like so::
+
+  ./bzr selftest --list-only
+
+This option can be combined with other selftest options (like -x) and
+filter patterns to understand their effect.
+
+
+Writing Tests
+=============
+
+In general tests should be placed in a file named test_FOO.py where 
+FOO is the logical thing under test. That file should be placed in the
+tests subdirectory under the package being tested.
+
+For example, tests for merge3 in bzrlib belong in bzrlib/tests/test_merge3.py.
+See bzrlib/tests/test_sampler.py for a template test script.
+
+Tests can be written for the UI or for individual areas of the library.
+Choose whichever is appropriate: if adding a new command, or a new command 
+option, then you should be writing a UI test.  If you are both adding UI
+functionality and library functionality, you will want to write tests for 
+both the UI and the core behaviours.  We call UI tests 'blackbox' tests
+and they are found in ``bzrlib/tests/blackbox/*.py``. 
+
+When writing blackbox tests please honour the following conventions:
+
+ 1. Place the tests for the command 'name' in
+    bzrlib/tests/blackbox/test_name.py. This makes it easy for developers
+    to locate the test script for a faulty command.
+
+ 2. Use the 'self.run_bzr("name")' utility function to invoke the command
+    rather than running bzr in a subprocess or invoking the
+    cmd_object.run() method directly. This is a lot faster than
+    subprocesses and generates the same logging output as running it in a
+    subprocess (which invoking the method directly does not).
+ 
+ 3. Only test the one command in a single test script. Use the bzrlib 
+    library when setting up tests and when evaluating the side-effects of
+    the command. We do this so that the library api has continual pressure
+    on it to be as functional as the command line in a simple manner, and
+    to isolate knock-on effects throughout the blackbox test suite when a
+    command changes its name or signature. Ideally only the tests for a
+    given command are affected when a given command is changed.
+
+ 4. If you have a test which does actually require running bzr in a
+    subprocess you can use ``run_bzr_subprocess``. By default the spawned
+    process will not load plugins unless ``--allow-plugins`` is supplied.
+
+
+Doctests
+--------
+
+We make selective use of doctests__.  In general they should provide 
+*examples* within the API documentation which can incidentally be tested.  We 
+don't try to test every important case using doctests -- regular Python
+tests are generally a better solution.
+
+Most of these are in ``bzrlib/doc/api``.  More additions are welcome.
+
+  __ http://docs.python.org/lib/module-doctest.html
+
+
+Skipping tests and test requirements
+------------------------------------
+
+In our enhancements to unittest we allow for some addition results beyond
+just success or failure.
+
+If a test can't be run, it can say that it's skipped.  This is typically
+used in parameterized tests - for example if a transport doesn't support
+setting permissions, we'll skip the tests that relating to that.  ::
+
+    try:
+        return self.branch_format.initialize(repo.bzrdir)
+    except errors.UninitializableFormat:
+        raise tests.TestSkipped('Uninitializable branch format')
+
+Raising TestSkipped is a good idea when you want to make it clear that the
+test was not run, rather than just returning which makes it look as if it
+was run and passed.
+
+A subtly different case is a test that should run, but can't run in the
+current environment.  This covers tests that can only run in particular
+operating systems or locales, or that depend on external libraries.  Here
+we want to inform the user that they didn't get full test coverage, but
+they possibly could if they installed more libraries.  These are expressed
+as a dependency on a feature so we can summarise them, and so that the
+test for the feature is done only once.  (For historical reasons, as of
+May 2007 many cases that should depend on features currently raise
+TestSkipped.)  The typical use is::
+
+    class TestStrace(TestCaseWithTransport):
+
+        _test_needs_features = [StraceFeature]
+
+which means all tests in this class need the feature.  The feature itself
+should provide a ``_probe`` method which is called once to determine if
+it's available.
+
+
+Known failures
+--------------
+
+Known failures are when a test exists but we know it currently doesn't
+work, allowing the test suite to still pass.  These should be used with
+care, we don't want a proliferation of quietly broken tests.  It might be
+appropriate to use them if you've committed a test for a bug but not the
+fix for it, or if something works on Unix but not on Windows.
+
+
+
+Essential Domain Classes
+########################
+
+Introducing the Object Model
+============================
+
+The core domain objects within the bazaar model are:
+
+* Transport
+
+* Branch
+
+* Repository
+
+* WorkingTree
+
+Transports are explained below. See http://bazaar-vcs.org/Classes/
+for an introduction to the other key classes.
+
+Using Transports
+================
+
+The ``Transport`` layer handles access to local or remote directories.
+Each Transport object acts like a logical connection to a particular
+directory, and it allows various operations on files within it.  You can
+*clone* a transport to get a new Transport connected to a subdirectory or
+parent directory.
+
+Transports are not used for access to the working tree.  At present
+working trees are always local and they are accessed through the regular
+Python file io mechanisms.
+
+Filenames vs URLs
+-----------------
+
+Transports work in URLs.  Take note that URLs are by definition only
+ASCII - the decision of how to encode a Unicode string into a URL must be
+taken at a higher level, typically in the Store.  (Note that Stores also
+escape filenames which cannot be safely stored on all filesystems, but
+this is a different level.)
+
+The main reason for this is that it's not possible to safely roundtrip a
+URL into Unicode and then back into the same URL.  The URL standard
+gives a way to represent non-ASCII bytes in ASCII (as %-escapes), but
+doesn't say how those bytes represent non-ASCII characters.  (They're not
+guaranteed to be UTF-8 -- that is common but doesn't happen everywhere.)
+
+For example if the user enters the url ``http://example/%e0`` there's no
+way to tell whether that character represents "latin small letter a with
+grave" in iso-8859-1, or "latin small letter r with acute" in iso-8859-2
+or malformed UTF-8.  So we can't convert their URL to Unicode reliably.
+
+Equally problematic if we're given a url-like string containing non-ascii
+characters (such as the accented a) we can't be sure how to convert that
+to the correct URL, because we don't know what encoding the server expects
+for those characters.  (Although this is not totally reliable we might still
+accept these and assume they should be put into UTF-8.)
+
+A similar edge case is that the url ``http://foo/sweet%2Fsour`` contains
+one directory component whose name is "sweet/sour".  The escaped slash is
+not a directory separator.  If we try to convert URLs to regular Unicode
+paths this information will be lost.
+
+This implies that Transports must natively deal with URLs; for simplicity
+they *only* deal with URLs and conversion of other strings to URLs is done
+elsewhere.  Information they return, such as from ``list_dir``, is also in
+the form of URL components.
+
+
+Core Topics
+###########
+
+Evolving Interfaces
+===================
 
 We have a commitment to 6 months API stability - any supported symbol in a
 release of bzr MUST NOT be altered in any way that would result in
@@ -72,128 +472,8 @@
 callers will at least get an AttributeError rather than weird results.
 
 
-Standard parameter types
-------------------------
-
-There are some common requirements in the library: some parameters need to be
-unicode safe, some need byte strings, and so on. At the moment we have
-only codified one specific pattern: Parameters that need to be unicode
-should be checked via ``bzrlib.osutils.safe_unicode``. This will coerce the
-input into unicode in a consistent fashion, allowing trivial strings to be
-used for programmer convenience, but not performing unpredictably in the
-presence of different locales.
-
-
-Copyright
----------
-
-The copyright policy for bzr was recently made clear in this email (edited
-for grammatical correctness)::
-
-    The attached patch cleans up the copyright and license statements in
-    the bzr source. It also adds tests to help us remember to add them
-    with the correct text.
-
-    We had the problem that lots of our files were "Copyright Canonical
-    Development Ltd" which is not a real company, and some other variations
-    on this theme. Also, some files were missing the GPL statements.
-    
-    I want to be clear about the intent of this patch, since copyright can
-    be a little controversial.
-    
-    1) The big motivation for this is not to shut out the community, but
-    just to clean up all of the invalid copyright statements.
-    
-    2) It has been the general policy for bzr that we want a single
-    copyright holder for all of the core code. This is following the model
-    set by the FSF, which makes it easier to update the code to a new
-    license in case problems are encountered. (For example, if we want to
-    upgrade the project universally to GPL v3 it is much simpler if there is
-    a single copyright holder). It also makes it clearer if copyright is
-    ever debated, there is a single holder, which makes it easier to defend
-    in court, etc. (I think the FSF position is that if you assign them
-    copyright, they can defend it in court rather than you needing to, and
-    I'm sure Canonical would do the same).
-    As such, Canonical has requested copyright assignments from all of the
-    major contributers.
-    
-    3) If someone wants to add code and not attribute it to Canonical, there
-    is a specific list of files that are excluded from this check. And the
-    test failure indicates where that is, and how to update it.
-    
-    4) If anyone feels that I changed a copyright statement incorrectly, just
-    let me know, and I'll be happy to correct it. Whenever you have large
-    mechanical changes like this, it is possible to make some mistakes.
-    
-    Just to reiterate, this is a community project, and it is meant to stay
-    that way. Core bzr code is copyright Canonical for legal reasons, and
-    the tests are just there to help us maintain that.
-
-
-Documentation
-=============
-
-When you change bzrlib, please update the relevant documentation for the
-change you made: Changes to commands should update their help, and
-possibly end user tutorials; changes to the core library should be
-reflected in API documentation.
-
-Commands
---------
-
-The docstring of a command is used by ``bzr help`` to generate help output
-for the command. The list 'takes_options' attribute on a command is used by
-``bzr help`` to document the options for the command - the command
-docstring does not need to document them. Finally, the '_see_also'
-attribute on a command can be used to reference other related help topics.
-
-NEWS file
----------
-
-If you make a user-visible change, please add a note to the NEWS file.
-The description should be written to make sense to someone who's just
-a user of bzr, not a developer: new functions or classes shouldn't be
-mentioned, but new commands, changes in behaviour or fixed nontrivial
-bugs should be listed.  See the existing entries for an idea of what
-should be done.
-
-Within each release, entries in the news file should have the most
-user-visible changes first.  So the order should be approximately:
-
- * changes to existing behaviour - the highest priority because the 
-   user's existing knowledge is incorrect
- * new features - should be brought to their attention
- * bug fixes - may be of interest if the bug was affecting them, and
-   should include the bug number if any
- * major documentation changes
- * changes to internal interfaces
-
-People who made significant contributions to each change are listed in
-parenthesis.  This can include reporting bugs (particularly with good
-details or reproduction recipes), submitting patches, etc.
-
-API documentation
------------------
-
-Functions, methods, classes and modules should have docstrings
-describing how they are used. 
-
-The first line of the docstring should be a self-contained sentence.
-
-For the special case of Command classes, this acts as the user-visible
-documentation shown by the help command.
-
-The docstrings should be formatted as reStructuredText_ (like this
-document), suitable for processing using the epydoc_ tool into HTML
-documentation.
-
-.. _reStructuredText: http://docutils.sourceforge.net/rst.html
-.. _epydoc: http://epydoc.sourceforge.net/
-
-
-
-Coding style
-============
+Coding Style Guidelines
+=======================
 
 Please write PEP-8__ compliant code.  
 
@@ -203,6 +483,17 @@
 __ http://www.python.org/peps/pep-0008.html
 
 
+Module Imports
+--------------
+
+* Imports should be done at the top-level of the file, unless there is
+  a strong reason to have them lazily loaded when a particular
+  function runs.  Import statements have a cost, so try to make sure
+  they don't run inside hot functions.
+
+* Module names should always be given fully-qualified,
+  i.e. ``bzrlib.hashcache`` not just ``hashcache``.
+
 
 Naming
 ------
@@ -224,7 +515,7 @@
 inconsistency if other people use the full name.
 
 
-Standard names
+Standard Names
 --------------
 
 ``revision_id`` not ``rev_id`` or ``revid``
@@ -331,7 +622,7 @@
 object, rather than the real class.
 
 
-Passing to other variables
+Passing to Other Variables
 ~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 It also is incorrect to assign ``ImportReplacer`` objects to other variables.
@@ -342,7 +633,30 @@
 variable, so some bugs are not detected right away.
 
 
-Writing output
+Getting Input
+=============
+
+Processing Command Lines
+------------------------
+
+bzrlib has a standard framework for parsing command lines and calling
+processing routines associated with various commands. See builtins.py
+for numerous examples.
+
+
+Standard Parameter Types
+------------------------
+
+There are some common requirements in the library: some parameters need to be
+unicode safe, some need byte strings, and so on. At the moment we have
+only codified one specific pattern: Parameters that need to be unicode
+should be checked via ``bzrlib.osutils.safe_unicode``. This will coerce the
+input into unicode in a consistent fashion, allowing trivial strings to be
+used for programmer convenience, but not performing unpredictably in the
+presence of different locales.
+
+
+Writing Output
 ==============
 
 (The strategy described here is what we want to get to, but it's not
@@ -379,89 +693,27 @@
 should be only in the command-line tool.
 
 
-Writing tests
-=============
-
-In general tests should be placed in a file named test_FOO.py where 
-FOO is the logical thing under test. That file should be placed in the
-tests subdirectory under the package being tested.
-
-For example, tests for merge3 in bzrlib belong in bzrlib/tests/test_merge3.py.
-See bzrlib/tests/test_sampler.py for a template test script.
-
-Tests can be written for the UI or for individual areas of the library.
-Choose whichever is appropriate: if adding a new command, or a new command 
-option, then you should be writing a UI test.  If you are both adding UI
-functionality and library functionality, you will want to write tests for 
-both the UI and the core behaviours.  We call UI tests 'blackbox' tests
-and they are found in ``bzrlib/tests/blackbox/*.py``. 
-
-When writing blackbox tests please honour the following conventions:
-
- 1. Place the tests for the command 'name' in
-    bzrlib/tests/blackbox/test_name.py. This makes it easy for developers
-    to locate the test script for a faulty command.
-
- 2. Use the 'self.run_bzr("name")' utility function to invoke the command
-    rather than running bzr in a subprocess or invoking the
-    cmd_object.run() method directly. This is a lot faster than
-    subprocesses and generates the same logging output as running it in a
-    subprocess (which invoking the method directly does not).
- 
- 3. Only test the one command in a single test script. Use the bzrlib 
-    library when setting up tests and when evaluating the side-effects of
-    the command. We do this so that the library api has continual pressure
-    on it to be as functional as the command line in a simple manner, and
-    to isolate knock-on effects throughout the blackbox test suite when a
-    command changes its name or signature. Ideally only the tests for a
-    given command are affected when a given command is changed.
-
- 4. If you have a test which does actually require running bzr in a
-    subprocess you can use ``run_bzr_subprocess``. By default the spawned
-    process will not load plugins unless ``--allow-plugins`` is supplied.
-
-
-Doctests
---------
-
-We make selective use of doctests__.  In general they should provide 
-*examples* within the API documentation which can incidentally be tested.  We 
-don't try to test every important case using doctests -- regular Python
-tests are generally a better solution.
-
-Most of these are in ``bzrlib/doc/api``.  More additions are welcome.
-
-  __ http://docs.python.org/lib/module-doctest.html
-
-
-Running tests
-=============
-Currently, bzr selftest is used to invoke tests.
-You can provide a pattern argument to run a subset. For example, 
-to run just the blackbox tests, run::
-
-  ./bzr selftest -v blackbox
-
-To skip a particular test (or set of tests), use the --exclude option
-(shorthand -x) like so::
-
-  ./bzr selftest -v -x blackbox  
-
-To list tests without running them, use the --list-only option like so::
-
-  ./bzr selftest --list-only
-
-This option can be combined with other selftest options (like -x) and
-filter patterns to understand their effect.
-
-
-Errors and exceptions
-=====================
-
-Errors are handled through Python exceptions.
+Handling Errors and Exceptions
+==============================
+
+Commands should return non-zero when they encounter circumstances that
+the user should really pay attention to - which includes trivial shell
+pipelines.
+
+Recommended values are:
+
+    0. OK.
+    1. Conflicts in merge-like operations, or changes are present in
+       diff-like operations. 
+    2. Unrepresentable diff changes (i.e. binary files that we cannot show 
+       a diff of).
+    3. An error or exception has occurred.
+
+Errors are handled through Python exceptions. Exceptions should be defined
+inside bzrlib.errors, so that we can see the whole tree at a glance.
 
 We broadly classify errors as either being either internal or not,
-depending on whether ``user_error`` is set or not.  If we think it's our
+depending on whether ``internal_error`` is set or not.  If we think it's our
 fault, we show a backtrace, an invitation to report the bug, and possibly
 other details.  This is the default for errors that aren't specifically
 recognized as being caused by a user error.  Otherwise we show a briefer
@@ -492,6 +744,118 @@
 final fullstop.  If long, they may contain newlines to break the text.
 
 
+Documenting Changes
+===================
+
+When you change bzrlib, please update the relevant documentation for the
+change you made: Changes to commands should update their help, and
+possibly end user tutorials; changes to the core library should be
+reflected in API documentation.
+
+NEWS File
+---------
+
+If you make a user-visible change, please add a note to the NEWS file.
+The description should be written to make sense to someone who's just
+a user of bzr, not a developer: new functions or classes shouldn't be
+mentioned, but new commands, changes in behaviour or fixed nontrivial
+bugs should be listed.  See the existing entries for an idea of what
+should be done.
+
+Within each release, entries in the news file should have the most
+user-visible changes first.  So the order should be approximately:
+
+ * changes to existing behaviour - the highest priority because the 
+   user's existing knowledge is incorrect
+ * new features - should be brought to their attention
+ * bug fixes - may be of interest if the bug was affecting them, and
+   should include the bug number if any
+ * major documentation changes
+ * changes to internal interfaces
+
+People who made significant contributions to each change are listed in
+parenthesis.  This can include reporting bugs (particularly with good
+details or reproduction recipes), submitting patches, etc.
+
+Commands
+--------
+
+The docstring of a command is used by ``bzr help`` to generate help output
+for the command. The list 'takes_options' attribute on a command is used by
+``bzr help`` to document the options for the command - the command
+docstring does not need to document them. Finally, the '_see_also'
+attribute on a command can be used to reference other related help topics.
+
+API Documentation
+-----------------
+
+Functions, methods, classes and modules should have docstrings
+describing how they are used. 
+
+The first line of the docstring should be a self-contained sentence.
+
+For the special case of Command classes, this acts as the user-visible
+documentation shown by the help command.
+
+The docstrings should be formatted as reStructuredText_ (like this
+document), suitable for processing using the epydoc_ tool into HTML
+documentation.
+
+.. _reStructuredText: http://docutils.sourceforge.net/rst.html
+.. _epydoc: http://epydoc.sourceforge.net/
+
+
+General Guidelines
+==================
+
+Copyright
+---------
+
+The copyright policy for bzr was recently made clear in this email (edited
+for grammatical correctness)::
+
+    The attached patch cleans up the copyright and license statements in
+    the bzr source. It also adds tests to help us remember to add them
+    with the correct text.
+
+    We had the problem that lots of our files were "Copyright Canonical
+    Development Ltd" which is not a real company, and some other variations
+    on this theme. Also, some files were missing the GPL statements.
+    
+    I want to be clear about the intent of this patch, since copyright can
+    be a little controversial.
+    
+    1) The big motivation for this is not to shut out the community, but
+    just to clean up all of the invalid copyright statements.
+    
+    2) It has been the general policy for bzr that we want a single
+    copyright holder for all of the core code. This is following the model
+    set by the FSF, which makes it easier to update the code to a new
+    license in case problems are encountered. (For example, if we want to
+    upgrade the project universally to GPL v3 it is much simpler if there is
+    a single copyright holder). It also makes it clearer if copyright is
+    ever debated, there is a single holder, which makes it easier to defend
+    in court, etc. (I think the FSF position is that if you assign them
+    copyright, they can defend it in court rather than you needing to, and
+    I'm sure Canonical would do the same).
+    As such, Canonical has requested copyright assignments from all of the
+    major contributers.
+    
+    3) If someone wants to add code and not attribute it to Canonical, there
+    is a specific list of files that are excluded from this check. And the
+    test failure indicates where that is, and how to update it.
+    
+    4) If anyone feels that I changed a copyright statement incorrectly, just
+    let me know, and I'll be happy to correct it. Whenever you have large
+    mechanical changes like this, it is possible to make some mistakes.
+    
+    Just to reiterate, this is a community project, and it is meant to stay
+    that way. Core bzr code is copyright Canonical for legal reasons, and
+    the tests are just there to help us maintain that.
+
+
+Miscellaneous Topics
+####################
 
 Debugging
 =========
@@ -505,11 +869,10 @@
 then bzr will go into pdb post-mortem mode when an unhandled exception
 occurs.
 
-If you send a SIGQUIT signal to bzr, which can be done by pressing C-\ on Unix,
-bzr will go into the debugger immediately.  You can continue execution by
-typing ``c``.  This can be disabled if necessary by setting the
-environment variable ``BZR_SIGQUIT_PDB=0``.
-
+If you send a SIGQUIT signal to bzr, which can be done by pressing
+Ctrl-\\ on Unix, bzr will go into the debugger immediately.  You can
+continue execution by typing ``c``.  This can be disabled if necessary
+by setting the environment variable ``BZR_SIGQUIT_PDB=0``.
 
 
 Jargon
@@ -521,56 +884,6 @@
     indexes into the branch's revision history.
 
 
-Transport
-=========
-
-The ``Transport`` layer handles access to local or remote directories.
-Each Transport object acts like a logical connection to a particular
-directory, and it allows various operations on files within it.  You can
-*clone* a transport to get a new Transport connected to a subdirectory or
-parent directory.
-
-Transports are not used for access to the working tree.  At present
-working trees are always local and they are accessed through the regular
-Python file io mechanisms.
-
-filenames vs URLs
------------------
-
-Transports work in URLs.  Take note that URLs are by definition only
-ASCII - the decision of how to encode a Unicode string into a URL must be
-taken at a higher level, typically in the Store.  (Note that Stores also
-escape filenames which cannot be safely stored on all filesystems, but
-this is a different level.)
-
-The main reason for this is that it's not possible to safely roundtrip a
-URL into Unicode and then back into the same URL.  The URL standard
-gives a way to represent non-ASCII bytes in ASCII (as %-escapes), but
-doesn't say how those bytes represent non-ASCII characters.  (They're not
-guaranteed to be UTF-8 -- that is common but doesn't happen everywhere.)
-
-For example if the user enters the url ``http://example/%e0`` there's no
-way to tell whether that character represents "latin small letter a with
-grave" in iso-8859-1, or "latin small letter r with acute" in iso-8859-2
-or malformed UTF-8.  So we can't convert their URL to Unicode reliably.
-
-Equally problematic if we're given a url-like string containing non-ascii
-characters (such as the accented a) we can't be sure how to convert that
-to the correct URL, because we don't know what encoding the server expects
-for those characters.  (Although this is not totally reliable we might still
-accept these and assume they should be put into UTF-8.)
-
-A similar edge case is that the url ``http://foo/sweet%2Fsour`` contains
-one directory component whose name is "sweet/sour".  The escaped slash is
-not a directory separator.  If we try to convert URLs to regular Unicode
-paths this information will be lost.
-
-This implies that Transports must natively deal with URLs; for simplicity
-they *only* deal with URLs and conversion of other strings to URLs is done
-elsewhere.  Information they return, such as from ``list_dir``, is also in
-the form of URL components.
-
-
 Unicode and Encoding Support
 ============================
 
@@ -637,49 +950,6 @@
 Use ``bzrlib.osutils.rmtree`` instead.
 
 
-Merge/review process
-====================
-
-If you'd like to propose a change, please post to the
-bazaar@lists.canonical.com list with a patch, bzr changeset, or link to a
-branch.  Please put '[patch]' in the subject so we can pick them out, and
-include some text explaining the change.  Remember to put an update to the NEWS
-file in your diff, if it makes any changes visible to users or plugin
-developers.  Please include a diff against mainline if you're giving a link to
-a branch.
-
-Please indicate if you think the code is ready to merge, or if it's just a
-draft or for discussion.  If you want comments from many developers rather than
-to be merged, you can put '[rfc]' in the subject lines.
-
-Anyone is welcome to review code.  There are broadly three gates for
-code to get in:
-
- * Doesn't reduce test coverage: if it adds new methods or commands,
-   there should be tests for them.  There is a good test framework
-   and plenty of examples to crib from, but if you are having trouble
-   working out how to test something feel free to post a draft patch
-   and ask for help.
-
- * Doesn't reduce design clarity, such as by entangling objects
-   we're trying to separate.  This is mostly something the more
-   experienced reviewers need to help check.
-
- * Improves bugs, features, speed, or code simplicity.
-
-Code that goes in should pass all three.
-
-If you read a patch please reply and say so.  We can use a numeric scale
-of -1, -0, +0, +1, meaning respectively "really don't want it in current
-form", "somewhat uncomfortable", "ok with me", and "please put it in".
-Anyone can "vote".   (It's not really voting, just a terse expression.)
-
-If something gets say two +1 votes from core reviewers, and no
-vetos, then it's OK to come in.  Any of the core developers can bring it
-into their integration branch, which I'll merge regularly.  (If you do
-so, please reply and say so.)
-
-
 C Extension Modules
 ===================
 
@@ -718,10 +988,12 @@
 maintaining the python code twice - once in the .pyx, and once in the .py,
 and no longer including the .py file.
 
-Making installers for OS Windows
+
+Making Installers for OS Windows
 ================================
 To build a win32 installer, see the instructions on the wiki page:
 http://bazaar-vcs.org/BzrWin32Installer
 
 
-:: vim: ft=rst tw=74 ai
+..
+   vim: ft=rst tw=74 ai

=== added file 'doc/developers/incremental-push-pull.txt'
--- doc/developers/incremental-push-pull.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/incremental-push-pull.txt	2007-05-09 15:36:06 +0000
@@ -0,0 +1,242 @@
+Incremental push/pull
+---------------------
+
+This use case covers pulling in or pushing out some number of revisions which
+is typically a small fraction of the number already present in the target
+repository. Pushing and pulling are defined as branch level operations for ease
+of interaction with VCS systems that have no repository abstraction (such as
+bzr-svn or GNU Arch) but within bzrlib's core they are currently the
+responsibility of the Repository object.
+
+Functional Requirements
+=======================
+
+A push or pull operation must:
+ * Copy all the data to reconstruct the selected revisions in the target
+   branch. This is the goal of push and pull after all.
+ * Reject corrupt data. As bzr has no innate mechanism for discarding corrupted
+   data, corrupted data should not be incorporated accidentally.
+
+Factors which should add work for push/pull
+===========================================
+
+ * Baseline overhead: The time to connect to both branches.
+ * Actual new data in the revisions being pulled (drives the amount of data to
+   move around, includes the commit messages etc)
+ * Number of revisions in the two repositories (scaling affects the
+   determination of what revisions to move around).
+
+Push/pull overview
+==================
+
+1. New data is identified in the source repository.
+2. That data is read from the source repository.
+3. The same data is verified and written to the target repository in such a
+   manner that its not visible to readers until its ready for use.
+
+New data identification
++++++++++++++++++++++++
+
+We have a single top level data object: revisions. Everything else is
+subordinate to revisions, so determining the revisions to propogate should be
+all thats needed. This depends on revisions with partial data - such as those
+with no signature - being flagged in some efficient manner.
+
+We could do this in two manners: determine revisions to sync and signatures to sync in two passes, or change the 'value' of a revision implicitly when the signature is different. E.g. by using merkle hash trees with the signature data a separate component the signatures will naturally be identified to sync.
+
+We want to only exchange data proportional to the number of new revisions and
+signatures in the system though. One way to achieve this for revisions is to
+walk the graph out from the desired tips until the surface area intersection is
+found. For signatures a set difference seems to be needed as there is no DAG of signatures: the presence of one has no implications on the presence of another, so a full pass over the set of signatures would be required to confirm no new signatures are needed (let alone replaced signatures).
+
+IFF we can determine 'new revisions' and 'new signatures' without full graph access then we can scale acceptable for push and pull.
+
+Ghosts are revisions which are not present in a particular repository. Filling ghosts refers to removing ghosts in the target repository when the ghost is present in the source repository. Filling ghosts can be either an explicit or implicit action. The common case is no ghosts.
+
+Set synchronisation approaches
+^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+
+A set synchronisation approach is one which synchronises two sets without
+regard for innate structure. This can be very efficient but requires adding a
+new node to be processed with every commit. Caching of the results of the
+various set based syncs I've seen is possible but because the data structures
+look different depending on the tip revision being synced up to the cache needs
+to be very complex. I recommend not using such an approach for the common case
+pull because of the failure to scale. We can use such an approach for
+synchronisation of new signatures and ghosts, which should be an explicit
+option in both cases.
+
+DAG synchronisation approaches
+^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+
+A DAG based approach to synchronistion is one that uses the DAG structure to
+determine the difference in present nodes. It can as a result operate from the
+tip of the DAG backwards. A dag based approach should allow incremental access
+to data and not require a full-graph scan for incremental operations.
+
+File level scaling
+^^^^^^^^^^^^^^^^^^
+
+We should read roughly as much of the revision level graph as is needed from
+each repository to determine the node difference.  If requested we should
+perform a detailed scan to pick up ghost revisions and revisions which have had
+signatures added. This should not be the default as it requires full history
+access in both cases.
+
+Expected file IO and access pattern:
+
+ * Common case: repo with many branches of one project, to the same.
+
+   1. Source and Target branch tips read.
+   2. Find the tip of each branch in their repo (will require reading some of
+      the revision graph but is typically near the end of the graph).
+   3. Read and parse increasing amounts of the revision graph until one is
+      found to be a subset of the other, or a complete list of revisions to be
+      transmitted is created.
+
+ * Uncommon cases: 
+   
+   1. Repositories with many projects or branches which are very old may
+      require reading a lot of unrelated graph data.
+
+   1. Initial push/pull scenarios should not require reading an entire graph.
+
+
+API scaling
+^^^^^^^^^^^
+
+ 1. Get branch tips.
+ 2. Determine one sided graph difference. To avoid obtaining a full graph over
+    the wire this needs to be done without reference to the full graph, and
+    with some logarthmic scaling algorithm. There are several already available
+    for this. 
+
+With ghost and new-signature detection:
+
+ * File IO access pattern will read the entire graph on the 'target' side - if
+   no ghosts are present then stop, otherwise seek the new revisions on the
+   source side with the regular algorithm and also explicitly search for the
+   ghost points from the target; plus a set difference search is needed on
+   signatures.
+
+ * Semantic level can probably be tuned, but as its also complex I suggest
+   deferring analysis for optimal behaviour of this use case.
+
+
+Data reading
+++++++++++++
+
+When transferring information about a revision the graph of data for the
+revision is walked: revision -> inventory, revision -> matching signature,
+inventory -> file ids:revision pairs.
+
+File level scaling
+^^^^^^^^^^^^^^^^^^
+
+As we're reading already committed data, as long as nothing is mutating data on
+disk reading should be race free. We will:
+
+ - read each revision object
+ - read the matching inventory delta
+ - attempt to read a signature object
+ - parse the inventory delta
+ - read the fileid:revisionid compressed chunk for each line in the inventory
+   delta
+
+Theres no point validating that the data read is valid, as transmission through to the client writing the data might invalidate it; we need to validate before we write.
+
+API scaling
+^^^^^^^^^^^
+
+Given that we have established the revisions needed, a single API call should
+suffice to obtain all data; the API should present the data in such an order
+that it can be validated as it arrives and thus not require large scale
+buffering on disk. Specifically each item of data should be validatable (e.g.
+for some file data we want the fileid:revisionid:validationhash + content).
+
+
+Data Verification and writing
++++++++++++++++++++++++++++++
+
+New data written to a repository should be completed intact when it is made
+visible. This suggests that either all the data for a revision must be made
+atomically visible (e.g. by renaming a single file) or the leaf nodes of the
+reference graph must become visible first.
+
+Data is referred to via the following graph:
+revision -> revision
+revision -> signature
+revision -> inventory
+inventory -> fileid:revisionid
+fileid:revisionid -> fileid:revisionid
+
+Data is verifiable via a different ordering:
+signature -> revision -> inventory -> fileid:revisionid texts.
+
+We dont gpg verify each revision today; this analysis only speaks to hash
+verification of contents.
+
+To validate a revision we need to validate the data it refers to. But to
+validate the contents of a revision we need the new texts in the inventory for
+the revision - to check a fileid:revisionid we need to know the expected sha1
+of the full text and thus also need to read the delta chain to construct the
+text as we accept it to determine if its valid. Providing separate validators
+for the chosen representation would address this.
+e.g: For an inventory entry FILEID:REVISIONID we store the validator of the
+full text :SHA1:. If we also stored the validator of the chosen disk
+representation (:DELTASHA1:) we could validate the transmitted representation
+without expanding the delta in the common case. If that failed we could expand
+the delta chain and try against the full text validator, and finally fail. As
+different delta generators might generate different deltas, :DELTASHA1: should
+not become part of the revision validator, only the inventory disk encoding. In
+a related manner a transmission format that allowed cheap validation of content
+without applying locally stored deltas would be advantageous because no local
+reads would be incurred to validate new content. For instance, always sending a
+full text for any file, possibly with a delta-chain when transmitting multiple
+revisionids of the file, would allow this. (git pack-files have this property).
+
+Overview summary
+^^^^^^^^^^^^^^^^
+
+A single-file local format would allow safe atomic addition of data while
+allowing optimisal transmission order of data. Failing this the validation of
+data should be tuned to not require reading local texts during data addition
+even in the presence of delta chains. We should have transmission-validators
+separate from content validators that allow validation of the delta-transmitted
+form of objects.
+
+File level scaling
+^^^^^^^^^^^^^^^^^^
+
+* Every new file text requires transmission and local serialisation.
+* Every commit requires transmission and storage of a revision, signature and inventory.
+
+Thus 4000 commits to a 50000 path tree of 10 files on averages requires (with
+knits) between 26 writes (2*(3+10)) and 80006 (2*(4000*10 + 3)) writes. In all
+cases there are 4000 * 13 distinct objects to record.
+
+Grouping data by fileid, content and metadata, gives the figures above.
+Data grouping:
+
+* File per full identifier (fileid:revisionid:meta|content): 104000
+* Delta-chain per object: object id count * constant overhead per object id 
+  (26 -> 80006)
+* Collation/pack file: 1
+
+Performance for these depends heavily on implementation:
+ - Using full ids we could name by validator or by id, giving best performance
+   that depends on either receiving data in validator order or in id order.
+ - using delta-chain per object we get least seek overhead and syscall overhead
+   if we recieve in topological order within the object id, and object ids in
+   lexical order.
+ - Using a collation/pack file we can stream it into place and validate as we go,
+   giving near ideal performance.
+
+API scaling
+^^^^^^^^^^^
+
+The api for writing new data recieved over the network will need to be geared
+to the transmission and local storage method. What we need is for the
+transmission method to reasonably closely match the desired write ordering
+locally. This suggests that once we decide on the best local storage means we
+should design the api.

=== added file 'doc/developers/index.txt'
--- doc/developers/index.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/index.txt	2007-05-08 04:14:01 +0000
@@ -0,0 +1,15 @@
+=======================
+Bazaar Developers Guide
+=======================
+
+-------------------
+Documentation index
+-------------------
+
+* `General HACKING HOWTO <HACKING.htm>`_
+
+  Guide to policies and tools for working on Bazaar itself.
+
+* `Performance roadmap <performance-roadmap.htm>`_
+
+  The roadmap for fixing performance in bzr over the next few releases.

=== added file 'doc/developers/performance-roadmap-rationale.txt'
--- doc/developers/performance-roadmap-rationale.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/performance-roadmap-rationale.txt	2007-05-07 17:49:30 +0000
@@ -0,0 +1,116 @@
+What should be in the roadmap?
+------------------------------
+
+A good roadmap provides a place for contributors to look for tasks, it
+provides users with a sense of when we will fix things that are
+affecting them, and it also allows us all to agree about where we are
+headed. So the roadmap should contain enough things to let all this
+happen.
+
+I think that it needs to contain the analysis work which is required, a
+list of the use cases to be optimised, the disk changes required, and
+the broad sense of the api changes required. It also needs to list the
+inter-dependencies between these things: we should aim for a large
+surface area of 'ready to be worked on' items, that makes it easy to
+improve performance without having to work in lockstep with other
+developers.
+
+Clearly the analysis step is an immediate bottleneck - we cannot tell if
+an optimisation for use case A is a pessimism for use case B until we
+have analysed both A and B. I propose that we complete the analysis of
+say a dozen core use cases end to end during the upcoming sprint in
+London. We should then be able to fork() for much of the detailed design
+work and regroup with disk and api changes shortly thereafter.
+
+I suspect that clarity of layering will make a big difference to
+developer parallelism, so another proposal I have is for us to look at
+the APIs for Branch and Repository in London in the light of what we
+have learnt over the last years.
+
+What should the final system look like, how is it different to what we have today?
+----------------------------------------------------------------------------------
+
+One of the things I like the most about bzr is its rich library API, and
+I've heard this from numerous other folk. So anything that will remove
+that should be considered a last resort.
+
+Similarly our relatively excellent cross platform support is critical
+for projects that are themselves cross platform, and thats a
+considerable number these days.
+
+And of course, our focus on doing the right thing is what differentiates
+us from some of the other VCS's, so we should be focusing on doing the
+right thing quickly :).
+
+What we have today though has grown organically in response to us
+identifying bottlenecks over several iterations of back end storage,
+branch metadata and the local tree representation. I think we are
+largely past that and able to describe the ideal characteristics of the
+major actors in the system - primarily Tree, Branch, Repository - based
+on what we have learnt.
+
+What use cases should be covered?
+---------------------------------
+
+My list of use cases is probably not complete - its just the ones I
+happen to see a lot :). I think each should be analysed comprehensively
+so we dont need to say 'push over the network' - its implied in the
+scaling analysis that both semantic and file operation latency will be
+considered.
+
+These use cases are ordered by roughly the ease of benchmarking, and the
+frequency of use. This ordering is so that when people are comparing bzr
+they are going to get use cases we have optimised; and so that as we
+speed things up our existing users will have the things they do the most
+optimised.
+
+ * status tree
+ * status subtree
+ * commit
+ * commit to a bound branch
+ * incremental push/pull
+ * log
+ * log path
+ * add
+ * initial push or pull [both to a new repo and an existing repo with
+   different data in it]
+ * diff tree
+ * diff subtree
+
+ * revert tree
+ * revert subtree
+ * merge from a branch
+ * merge from a bundle
+ * annotate
+ * create a bundle against a branch
+ * uncommit
+ * missing
+ * update
+ * cbranch
+
+how is development on the roadmap coordinated?
+----------------------------------------------
+
+I think we should hold regular get-togethers (on IRC) to coordinate on
+our progress, because this is a big task and its a lot easier to start
+helping out some area which is having trouble if we have kept in contact
+about each areas progress. This might be weekly or fortnightly or some
+such.
+
+we need a shared space to record the results of the analysis and the
+roadmap as we go forward. Given that we'll need to update these as new
+features are considered, I propose that we use doc/design as a working
+space, and as we analyse use cases we include them in there - including
+the normal review process for each patch. We also need documentation
+about doing performance tuning - not the minutiae, though that is
+needed, but about how to effective choose things to optimise which will
+give the best return on time spent - that is what the roadmap should
+help with, but this looks to be a large project and an overview will be
+of great assistance I think. We want to help everyone that wishes to
+contribute to performance to do so effectively.
+
+Finally, its important to note that coding is not the only contribution
+- testing, giving feedback on current performance, helping with the
+analysis are all extremely important tasks too and we probably want to
+have clear markers of where that should be done to encourage such
+contributions.

=== added file 'doc/developers/performance-roadmap.txt'
--- doc/developers/performance-roadmap.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/performance-roadmap.txt	2007-05-09 15:36:06 +0000
@@ -0,0 +1,15 @@
+==========================
+Bazaar Performance Roadmap
+==========================
+
+About the performance roadmap
+#############################
+
+.. include:: performance-roadmap-rationale.txt
+
+Analysis of use cases
+#####################
+
+.. include:: performance-use-case-analysis.txt
+
+.. include:: incremental-push-pull.txt

=== added file 'doc/developers/performance-use-case-analysis.txt'
--- doc/developers/performance-use-case-analysis.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/performance-use-case-analysis.txt	2007-05-09 15:36:06 +0000
@@ -0,0 +1,108 @@
+Analysing a specific use case
+-----------------------------
+
+The analysis of a use case needs to provide as outputs:
+ * The functional requirements that the use case has to satisfy.
+ * The file level operations and access patterns that will give the best
+   performance.
+ * A low friction API which will allow the use case to be implemented.
+ * The release of bzr (and thus the supported features) for which the analysis
+   was performed. The feature set of bzr defines the access patterns and data
+   required to implement any use case. So when we add features, their design
+   changes the requirements for the parts of the system they alter, so we need
+   to re-analyse use cases when bzr's feature set changes. If future plans are
+   considered in the analysis with the intention of avoiding rework, these
+   should also be mentioned.
+
+Performing the analysis
+-----------------------
+
+The analysis needs to be able to define the characteristics of the
+involved disk storage and APIs. That means we need to examine the data
+required for the operation, in what order it is required, on both the
+read and write sides, and how that needs to be presented to be
+consistent with our layering.
+
+As a quick example: 'annotation of a file requires the file id looked up
+from the tree, the basis revision id from the tree, and then the text of
+that fileid-revisionid pair along with the creating revision id
+allocated to each line, and the dotted revision number of each of those
+revision ids.' All three of our key domain objects are involved here,
+but we haven't defined any characteristics of the api or disk facilities
+yet. We could then do that by saying something like 'the file-id lookup
+should degrade gracefully as trees become huge. The tree basis id should
+be constant time. Retrieval of the annotated text should be roughly
+constant for any text of the same size regardless of the number of
+revisions contributing to its content. Mapping of the revision ids to
+dotted revnos could be done as the text is retrieved, but its completely
+fine to post-process the annotated text to obtain dotted-revnos.'
+
+What factors should be considered?
+----------------------------------
+
+Obviously, those that will make for an extremely fast system :). There
+are many possible factors, but the ones I think are most interesting to
+design with are:
+
+- baseline overhead:
+
+   - The time to get bzr ready to begin the use case.
+
+- scaling: how does performance change when any of the follow aspects
+   of the system are ratched massively up or down:
+
+   - number of files/dirs/symlinks/subtrees in a tree (both working and 
+     revision trees)
+   - size of any particular file
+   - number of elements within a single directory
+   - length of symlinks
+   - number of changes to any file over time
+     (subordinately also the number of merges of the file)
+   - number of commits in the ancestry of a branch
+     (subordinately also the number of merges)
+   - number of revisions in a repository
+   - number of fileids in a repository
+   - number of ghosts in a given graph (revision or per-file)
+   - number of branches in a repository
+   - number of concurrent readers for a tree/branch/repository
+   - number of concurrent writers for objects that support that.
+   - latency to perform file operations (e.g. slow disks, network file systems,
+     our VFS layer and FTP/SFTP/etc)
+   - bandwidth to the disk storage
+   - latency to perform semantic operations (hpss specific)
+   - bandwidth when performing semantic operations.
+- locality of reference: If an operation requires data that is located
+   within a small region at any point, we often get better performance 
+   than with an implementation of the same operation that requires the
+   same amount of data but with a lower locality of reference. Its 
+   fairly tricky to add locality of reference after the fact, so I think
+   its worth considering up front.
+
+Using these factors, to the annotate example we can add that its
+reasonable to do two 'semantic' round trips to the local tree, one to
+the branch object, and two to the repository. In file-operation level
+measurements, in an ideal world there would be no more than one round
+trip for each semantic operation. What there must not be is one round
+trip per revision involved in the revisionid->dotted number mapping, nor
+per each revision id attributed to a line in the text. 
+
+Not all the items mentioned above are created equal. The analysis should
+include the parameters considered and the common case values for each - the
+optimisation should be around the common cases not around the exceptions.
+
+For instance, we have a smart server now; file level operations are relatively
+low latency and we should use that as the common case. At this point we intend
+to preserve the performance of the dumb protocol networking, but focus on
+improving network performance via the smart server and thus escape the
+file-level operation latency considerations.
+
+Many performance problems only become visible when changing the scaling knobs
+upwards to large trees. On small trees its our baseline performance that drives
+incremental improvements; on large trees its the amount of processing per item
+that drives performance. A significant goal therefore is to keep the amouont of
+data to be processed under control. Ideally we can scale in a sublinear fashion
+for all operations, but we MUST NOT scale even linearly for operations that
+invoke a latency multiplier. For example, reading a file on disk requires
+finding the inode for the file, then the block with the data and returning the
+contents. Due to directory grouping logic we pay a massive price to read files
+if we do not group the reads of files within the same directory.

=== modified file 'doc/index.txt'
--- doc/index.txt	2007-04-12 09:21:48 +0000
+++ doc/index.txt	2007-05-08 04:14:01 +0000
@@ -51,7 +51,7 @@
 
     How to run a smart HTTP server to allow remote access to Bazaar branches.
 
-* `Guidelines for modifying bzr <HACKING.htm>`_
+* `Guidelines for modifying bzr <developers/index.htm>`_
 
     Guide to policies and tools for working on Bazaar itself.
 

=== added file 'doc/shared_repository_layouts.txt'
--- doc/shared_repository_layouts.txt	1970-01-01 00:00:00 +0000
+++ doc/shared_repository_layouts.txt	2007-05-07 17:49:30 +0000
@@ -0,0 +1,369 @@
+=======================
+SharedRepositoryLayouts
+=======================
+
+.. sectnum::
+
+Bazaar is designed to give you flexibility in how you layout branches inside a shared repository. 
+This flexibility allows users to tailor Bazaar to their workflow,
+but it also leads to questions about what is a "good" layout.
+We present some alternatives and give some discussion about the benefits of each.
+
+One key point which should be mentioned is that any good layout should somehow highlight
+what branch a "general" user should grab. In SVN this is deemed the "``trunk/``" branch,
+and in most of the layouts this naming convention is preserved. Some would call this
+"``mainline``" or "``dev``", and people from CVS often refer to this as "``HEAD``".
+
+.. contents::
+
+
+"SVN-Style" (``trunk/``, ``branches/``)
+=======================================
+
+Most people coming from SVN will be familiar with their "standard" project layout. 
+Which is to layout the repository as::
+
+  repository/       # Overall repository
+   +- trunk/        # The mainline of development
+   +- branches/     # A container directory
+   |   +- foo/      # Branch for developing feature foo
+   |     ...
+   +- tags/         # Container directory
+       +- release-X # A branch specific to mark a given release version
+          ...
+
+With Bazaar, that is a perfectly reasonable layout. 
+It has the benefit of being familiar to people coming from SVN, 
+and making it clear where the development focus is.
+
+When you have multiple projects in the same repository, 
+the SVN layout is a little unclear what to do.
+
+
+``project/trunk``
+~~~~~~~~~~~~~~~~~
+
+The preferred method for SVN seems to be to give each project a top level directory for a layout like::
+
+  repository/            # Overall repository
+   +- project1/          # A container directory
+   |   +- trunk/         # The mainline of development of project1
+   |   +- branches/      # A container directory
+   |       +- foo/       # Branch for developing feature foo of project1
+   |         ...
+   |
+   +- project2/          # Container for project2
+       +- trunk/         # Mainline for project2
+       +- branches/      # Container for project2 branches
+
+
+This also works with Bazaar. 
+However, with Bazaar repositories are cheap to create 
+(a simple ``bzr init-repo`` away), and their primary benefit is when the
+branches share a common ancestry.
+
+So the preferred way for Bazaar would be::
+
+    project1/          # A repository for project1
+     +- trunk/         # The mainline of development of project1
+     +- branches/      # A container directory
+         +- foo/       # Branch for developing feature foo of project1
+           ...
+    
+    project2/          # A repository for project2
+     +- trunk/         # Mainline for project2
+     +- branches/      # Container for project2 branches
+
+
+``trunk/project``
+~~~~~~~~~~~~~~~~~
+
+There are also a few projects who use this layout in SVN::
+
+  repository/             # Overall repository
+    +- trunk/             # A container directory
+    |   +- project1       # Mainline for project 1
+    |   +- project2       # Mainline for project 2
+    |         ...
+    |
+    +- branches/          # Container
+        +- project1/      # Container (?)
+        |   +- foo        # Branch 'foo' of project1
+        +- project2/
+            +- bar        # Branch 'bar' of project2
+
+
+A slight variant is::
+
+  repository/             # Overall repository
+    +- trunk/             # A container directory
+    |   +- project1       # Mainline for project 1
+    |   +- project2       # Mainline for project 2
+    |         ...
+    |
+    +- branches/          # Container
+        +- project1-foo/  # Branch 'foo' of project1
+        +- project2-bar/  # Branch 'bar' of project2
+
+I believe the reason for this in SVN, is so that someone 
+can checkout all of "``trunk/``" and get the all the mainlines for all projects.
+
+This layout can be used for Bazaar, but it is not generally recommended.
+
+ 1) ``bzr branch/checkout/get`` is a single branch at a time. 
+    So you don't get the benefit of getting all mainlines with a single command. [1]_
+
+ 2) It is less obvious of whether ``repository/trunk/foo`` is the ``trunk`` of project 
+    ``foo`` or it is just the ``foo`` directory in the ``trunk`` branch.
+    Some of this confusion is due to SVN, because it uses the same "namespace" 
+    for files in a project that it uses for branches of a project.
+    In Bazaar, there is a clear distinction of what files make up a project, versus
+    the location of the Branch. (After all, there is only one ``.bzr/`` directory per branch,
+    versus many ``.svn/`` directories in the checkout).
+
+.. [1] Note: `NestedTreeSupport`_ can provide a way to create "meta-projects" which
+    aggregate multiple projects regardless of the repository layout.
+    Letting you ``bzr checkout`` one project, and have it grab all the necessary
+    sub-projects.
+
+.. _NestedTreeSupport: http://bazaar-vcs.org/NestedTrees
+
+
+Nested Style (``project/branch/sub-branch/``)
+=============================================
+
+Another style with Bazaar, which is not generally possible in SVN
+is to have branches nested within each-other.
+This is possible because Bazaar supports (and recommends) creating repositories
+with no working trees (``--no-trees``).
+With a ``--no-trees`` repository, because the working files are not intermixed with
+your branch locations, you are free to put a branch in whatever namespace you want.
+
+One possibility is::
+
+  project/             # The overall repository, *and* the project's mainline branch
+   + joe/              # Developer Joe's primary branch of development
+   |  +- feature1/     # Developer Joe's feature1 development branch
+   |  |   +- broken/   # A staging branch for Joe to develop feature1
+   |  +- feature2/     # Joe's feature2 development branch
+   |    ...
+   + barry/            # Barry's development branch
+   |  ...
+   + releases/
+      +- 1.0/
+          +- 1.1.1/
+
+The idea with this layout is that you are creating a hierarchical layout for branches. 
+Where changes generally flow upwards in the namespace. It also gives people a little
+corner of the namespace to work on their stuff.
+One nice feature of this layout, is it makes branching "cheaper" because it gives you
+a place to put all the mini branches without cluttering up the global ``branches/`` namespace.
+
+The other power of this is that you don't have to repeat yourself when specifying more detail in the
+branch name.
+
+For example compare::
+
+  bzr branch http://host/repository/project/branches/joe-feature-foo-bugfix-10/
+
+Versus::
+  
+  bzr branch http://host/project/joe/foo/bugfix-10
+
+
+Also, if you list the ``repository/project/branches/`` directory you might see something like::
+
+  barry-feature-bar/
+  barry-bugfix-10/
+  barry-bugfix-12/
+  joe-bugfix-10/
+  joe-bugfix-13/
+  joe-frizban/
+
+Versus having these broken out by developer. 
+If the number of branches are small, ``branches/`` has the nice advantage
+of being able to see all branches in a single view.
+If the number of branches is large, ``branches/`` has the distinct disadvantage
+of seeing all the branches in a single view (it becomes difficult to find the
+branch you are interested in, when there are 100 branches to look through).
+
+Nested branching seems to scale better to larger number of branches.
+However, each individual branch is less discoverable. 
+(eg. "Is Joe working on bugfix 10 in his feature foo branch, or his feature bar branch?")
+
+One other small advantage is that you can do something like::
+
+   bzr branch http://host/project/release/1/1/1
+  or
+   bzr branch http://host/project/release/1/1/2
+
+To indicate release 1.1.1 and 1.1.2. This again depends on how many releases you have 
+and whether the gain of splitting things up outweighs the ability to see more at a glance.
+
+
+Sorted by Status (``dev/``, ``merged/``, ``experimental/``)
+===========================================================
+
+One other way to break up branches is to sort them by their current status.
+So you would end up with a layout something like::
+
+  project/               # Overall layout
+   +- trunk/             # The development focus branch
+   +- dev/               # Container directory for in-progress work
+   |   +- joe-feature1   # Joe's current feature-1 branch
+   |   +- barry-bugfix10 # Barry's work for bugfix 10
+   |    ...
+   +- merged/            # Container indicating these branches have been merged
+   |   +- bugfix-12      # Bugfix which has already been merged.
+   +- abandonded/        # Branches which are considered 'dead-end'
+
+
+This has a couple benefits and drawbacks.
+It lets you see what branches are actively being developed on, which is usually
+only a small number, versus the total number of branches ever created.
+Old branches are not lost (versus deleting them), but they are "filed away",
+such that the more likely you are to want a branch the easier it is to find. 
+(Conversely, older branches are likely to be harder to find).
+
+The biggest disadvantage with this layout, is that branches move around.
+Which means that if someone is following the ``project/dev/new-feature`` branch,
+when it gets merged into ``trunk/`` suddenly ``bzr pull`` doesn't mirror the branch
+for them anymore because the branch is now at ``project/merged/new-feature``.
+There are a couple ways around this. One is to use HTTP redirects to point people
+requesting the old branch to the new branch. ``bzr`` >= 0.15 will let users know
+that ``http://old/path redirects to http://new/path``. However, this doesn't help
+if people are accessing a branch through methods other than HTTP (SFTP, local filesystem, etc).
+
+It would also be possible to use a symlink for temporary redirecting (as long as the symlink
+is within the repository it should cause little trouble). However eventually you want to
+remove the symlink, or you don't get the clutter reduction benefit.
+Another possibility instead of a symlink is to use a ``BranchReference``. It is currently
+difficult to create these through the ``bzr`` command line, but if people find them useful
+that could be changed.
+This is actually how `Launchpad`_ allows you to ``bzr checkout https://launchpad.net/bzr``.
+Effectively a ``BranchReference`` is a symlink, but it allows you to reference any other URL.
+If it is extended to support relative references, it would even work over http, sftp, 
+and local paths.
+
+.. _Launchpad: https://launchpad.net
+
+
+Sorted by date/release/etc (``2006-06/``, ``2006-07/``, ``0.8/``, ``0.9``)
+==========================================================================
+
+Another method of allowing some scalability while also allowing the
+browsing of "current" branches. Basically, this works on the assumption 
+that actively developed branches will be "new" branches, and older branches
+are either merged or abandoned.
+
+Basically the date layout looks something like::
+
+  project/                # Overall project repository
+   +- trunk/              # General mainline
+   +- 2006-06/            # containing directory for branches created in this month
+   |   +- feature1/       # Branch of "project" for "feature1"
+   |   +- feature2/       # Branch of "project" for "feature2"
+   +- 2005-05/            # Containing directory for branches create in a different month
+       +- feature3/
+       ...
+
+This answers the question "Where should I put my new branch?" very quickly.
+If a feature is developed for a long time, it is even reasonable to copy a
+branch into the newest date, and continue working on it there.
+Finding an active branch generally means going to the newest date, and
+going backwards from there. (A small disadvantage is that most directory
+listings sort oldest to the top, which may mean more scrolling).
+If you don't copy old branches to newer locations, it also has the disadvantage
+that searching for a branch may take a while.
+
+Another variant is by release target::
+
+  project/          # Overall repository
+   +- trunk/        # Mainline development branch
+   +- releases/     # Container for release branches
+   |   +- 0.8/      # The branch for release 0.8
+   |   +- 0.9/      # The branch for release 0.9
+   +- 0.8/          # Container for branches targeting release 0.8
+   |   +- feature1/ # Branch for "feature1" which is intended to be merged into 0.8
+   |   +- feature2/ # Branch for "feature2" which is targeted for 0.8
+   +- 0.9/
+       +- feature3/ # Branch for "feature3", targeted for release 0.9
+
+
+Some possible variants include having the ``0.9`` directory imply
+that it is branched *from* 0.9 rather than *for* 0.9, or having the ``0.8/release``
+as the official release 0.8 branch.
+
+The general idea is that by targeting a release, you can look at what branches are
+waiting to be merged. It doesn't necessarily give you a good idea of what the
+state of the branch (is it in development or finished awaiting review).
+It also has a history-hiding effect, and otherwise has the same benefits 
+and deficits as a date-based sorting. 
+
+
+Simple developer naming (``project/joe/foo``, ``project/barry/bar``)
+====================================================================
+
+Another possibly layout is to give each developer a directory, and then
+have a single sub-directory for branches. Something like::
+
+  project/      # Overall repository
+   +- trunk/    # Mainline branch
+   +- joe/      # A container for Joe's branches
+   |   +- foo/  # Joe's "foo" branch of "project"
+   +- barry/
+       +- bar/  # Barry's "bar" branch of "project"
+
+The idea is that no branch is "nested" underneath another one, just that each developer
+has his/her branches grouped together.
+
+A variant which is used by `Launchpad`_ is::
+
+  repository/
+   +- joe/             # Joe's branches
+   |   +- project1/    # Container for Joe's branches of "project1"
+   |   |   +- foo/     # Joe's "foo" branch of "project1"
+   |   +- project2/    # Container for Joe's "project2" branches
+   |       +- bar/     # Joe's "bar" branch of "project2"
+   |        ...
+   |
+   +- barry/
+   |   +- project1/    # Container for Barry's branches of "project1"
+   |       +- bug-10/  # Barry's "bug-10" branch of "project1"
+   |   ...
+   +- group/
+       +- project1/
+           +- trunk/   # The main development focus for "project1"
+
+
+This lets you easily browse what each developer is working on. Focus branches
+are kept in a "group" directory, which lets you see what branches the "group"
+is working on.
+
+This keeps different people's work separated from each-other, but also makes it
+hard to find "all branches for project X". `Launchpad`_ compensates for this
+by providing a nice web interface with a database back end, which allows a
+"view" to be put on top of this layout.
+This is closer to the model of people's home pages, where each person has a
+"``~/public_html``" directory where they can publish their own web-pages.
+In general, though, when you are creating a shared repository for centralization
+of a project, you don't want to split it up by person and then project.
+Usually you would want to split it up by project and then by person.
+
+
+Summary
+=======
+
+In the end, no single naming scheme will work for everyone. It depends a lot on
+the number of developers, how often you create a new branch, what sort of
+lifecycles your branches go through. Some questions to ask yourself:
+
+  1) Do you create a few long-lived branches, or do you create lots of "mini" feature branches
+     (Along with this is: Would you *like* to create lots of mini feature branches, but can't
+     because they are a pain in your current VCS?)
+
+  2) Are you a single developer, or a large team?
+
+  3) If a team, do you plan on generally having everyone working on the same branch at the same
+     time? Or will you have a "stable" branch that people are expected to track.
+

